// ****************************************************************************
//
// COPYRIGHT 2012 AyDeeKay LLC - All Rights Reserved
// No part of this document can be used, copied, transmitted, or modified by
// any existing or to be invented means without express approval of AyDeeKay
// LLC.
//
// ****************************************************************************
//
// File:
//   usesame.h
//
// Description:  
//   Definitions and function declarations for Chaviero IC
//
// ****************************************************************************
//
// REVISION HISTORY
//
// Version   Date:        By:          Description:
// V0.02.00  08/08/2012   David Chen   NVIC_SetPriority modified
// V0.01.00  04/18/2012   Scott Kee    Unify various versions into one file
//
// **************************************************************************** 


#ifndef __USesame_H__
#define __USesame_H__

// ********************************************************************************************************************
// IRQ enumeration - must be defined before core_cm0.h loaded
// ********************************************************************************************************************

/// Interrupt table:
// enum for IRQ names

typedef enum IRQn
{
  //******  Cortex-M0 Processor Exceptions Numbers ****************************
  NonMaskableInt_IRQn   = -14,    // Non Maskable Interrupt                             
  HardFault_IRQn        = -13,    // Hard Fault Interrupt                     
  SVCall_IRQn           = -5,     // SV Call Interrupt                       
  PendSV_IRQn           = -2,     // Pend SV Interrupt                       
  SysTick_IRQn          = -1,     // System Tick Interrupt                   

  //*****  CM0IKMCU Cortex-M0 specific Interrupt Numbers **********************
  BrownOut_IRQn     	= 0,      // Brown out alarm                             
  ClkMon_IRQn       	= 1,      // Clock monitor alarm
  SS_IRQn         		= 2,      // Not assigned
  PIN_IRQn          	= 3,      // Pin change detected
  RFRE_IRQn         	= 4,      // Rising Edge RF interrupt
  RFFE_IRQn         	= 5,      // Falling Edge RF interrupt
  I2C_Collision_IRQn	= 6,      // Collision processing - I2C
  I2C_IRQn              = 7,      // I2C interrupt
  UART_IRQn             = 8,      // UART interrupt
  LIN_IRQn              = 9,      // LIN (CAN) interrupt
  SPI_IRQn              = 10,     // SPI interrupt
  IRQ11_IRQn            = 11,     // Not assigned
  RFMSG_IRQn            = 12,     // RF Message
  IRQ13_IRQn            = 13,     // Not asssigned
  IRQ14_IRQn            = 14,     // Not asssigned
  IRQ15_IRQn            = 15,     // Not asssigned
  TIMER0_IRQn           = 16,     // Timer 0
  TIMER1_IRQn           = 17,     // Timer 1
  TIMER2_IRQn           = 18,     // Timer 2
  WATCHDOG_IRQn         = 19      // Watchdog timer
} IRQn_Type;



#include "clough_core.h"
#include <stdint.h>

#define	FALSE 	0x00
#define	TRUE	0x01


typedef struct{
  uint16_t counter;
  uint16_t random;
  uint8_t buttoms;
  uint32_t id;
}RF_MSG;

typedef union
{ 
  __IO uint16_t HWORD;
  __IO uint8_t  BYTE[2];
} PirCtrlType;

typedef union
{
  __IO uint32_t WORD;
  __IO uint16_t HWORD[2];
  __IO uint8_t  BYTE[4];
} TxConfigType;

typedef union
{
  __IO uint16_t HWORD;
  __IO uint8_t  BYTE[2];
} TxDataBufType;

typedef union
{
  __IO uint16_t HWORD;
  __IO uint8_t  BYTE[2];
} TxCtrlAType;

typedef union
{
  __IO uint32_t WORD;
  __IO uint16_t HWORD[2];
  __IO uint8_t  BYTE[4];  
} TxCtrlBType;

typedef union
{
  __IO uint16_t HWORD;
  __IO uint8_t  BYTE[2];
} GpioCtrlType;

typedef union
{
  __IO uint16_t HWORD;
  __IO uint8_t  BYTE[2];
} GpioType;

typedef union
{
  __IO uint32_t WORD;
  __IO uint16_t HWORD[2];
  __IO uint8_t  BYTE[4];
} GpioIntType;

typedef union
{
  __IO uint32_t WORD;
  __IO uint16_t HWORD[2];
  __IO uint8_t  BYTE[4];
} TrimWordType;

typedef union
{
  __IO uint16_t HWORD[5];
  __IO uint8_t  BYTE[10];
} RxData;

typedef union
{ 
  __IO uint32_t WORD[2];
  __IO uint16_t HWORD[4];
  __IO uint8_t  BYTE[8];
} LinDataType;


typedef   __IO uint8_t  reg8;
typedef   __IO uint16_t  reg16;

//System Timer (SysTick) definitions
#define SYSTICK_CLK_MCU     0x04  //SysTick timer clock source: processor clock.
#define SYSTICK_CLK_REF     0x00  //SysTick timer clock source: reference clock.
#define SYSTICK_INT_EN      0x02  //SysTick exception request: counting down to zero to asserts the SysTick exception request.
#define SYSTICK_INT_DIS     0x00  //SysTick exception request: counting down to zero does not assert the SysTick exception request.
#define SYSTICK_COUNTER_EN  0x01  //Enable the counter: counter enabled.
#define SYSTICK_COUNTER_DIS 0x00  //Enable the counter: counter disabled.

// FAP = Fast Access Peripheral
// SAP = Slow Access Peripheral

#define	PMU_BASE	(ASIC_7B_BASE + 0x00)
#define	I2C_BASE	(ASIC_7B_BASE + 0x08)
#define	UART_BASE	(ASIC_7B_BASE + 0x10)
#define	SPI_BASE	(ASIC_7B_BASE + 0x1C)
#define	RF_BASE		(ASIC_7B_BASE + 0x20)
#define	LIN_BASE	(ASIC_7B_BASE + 0x30)
#define	PWM_BASE	(ASIC_7B_BASE + 0x48)
#define	ADC_BASE	(ASIC_7B_BASE + 0x50)
#define	GPIO_FAP_BASE	(ASIC_7B_BASE + 0x60)

#define	BLOCK_BASE	(ASIC_7B_BASE + 0x80)

#define	GPIO_SAP_BASE	(ASIC_16B_BASE + 0x0000)
#define	RF_SAP_BASE	(ASIC_16B_BASE + 0x1000)
#define	TRIM_SAP_BASE	(ASIC_16B_BASE + 0x8004)
#define ADC_SAP_BASE    (ASIC_16B_BASE + 0x8008)



#define PMUCLK   ((__IO uint8_t *) (PMU_BASE + 0x00))	//Power Management - Clock
#define	CLKDIV1		0x00		//Clock Dividers
#define	CLKDIV2		0x40
#define	CLKDIV4		0x80
#define	CLKDIV8		0xC0
#define	XTMON_ON	0x20		//Crystal osc. monitor 
#define	XTMON_OFF	0x00
#define	RCMON_ON	0x10		//RC osc. monitor 
#define	RCMON_OFF	0x00
#define	XTON		0x08		//Crystal clock enable
#define	XTOFF		0x00		//Crystal Clock off
#define	RCON		0x04		//RC clock enable
#define	RCOFF		0x00		//RC Clock off
#define	RC10KHZ		0x00		//Clock source = RC AUX 10KHz
#define	RC10MHZ		0x01		//Clock source = RC 10MHz
#define	XTCLOCK		0x02		//Clock source = XT 
#define	RCAUX		0x03		//Clock fault --> upon reset goes to 10KHz (0x00)

/**
* @brief Select Clock Divider
*
* @param divider specifies the clock divider. Options:
* @param CLKDIV1 = Divide by 1	
* @param CLKDIV2 = Divide by 2
* @param CLKDIV4 = Divide by 4
* @param CLKDIV8 = Divide by 8
* @return none
*
* 
*/
static __INLINE void CLK_Divider( uint8_t divider)
{
	*PMUCLK = (*PMUCLK & (0x3F)) | divider;
}

/**
* @brief Read Crystal Clock monitor
*
* @param none
* @return XTMON_ON or XTMON_OFF
*
* 
*/
static __INLINE uint8_t CLK_CrystalMonitor( void )
{
	return ( *PMUCLK & XTMON_ON );
}

/**
* @brief Read RC Clock monitor
*
* @param none
* @return RCMON_ON or RCMON_OFF
*
* 
*/
static __INLINE uint8_t CLK_RcMonitor( void )
{
	return ( *PMUCLK & RCMON_ON );
}


/**
* @brief Control Crystal Clock 
*
* @param mode:
* @param XTON
* @param XTOFF
*
* 
*/
static __INLINE void CLK_CrystalControl( uint8_t mode )
{
	if ( mode == XTON ) *PMUCLK |= XTON; else *PMUCLK &= ~XTON;
}

/**
* @brief Control RC Clock 
*
* @param mode:
* @param RCON
* @param RCOFF
*
* 
*/
static __INLINE void CLK_RcControl( uint8_t mode )
{
	if ( mode == RCON ) *PMUCLK |= RCON; else *PMUCLK &= ~RCON;
}

/**
* @brief Select Clock Source
*
* @param source:
* @param 	RC10KHZ - Clock source = RC AUX 10KHz
* @param 	RC10MHz - Clock source = RC 10MHz
* @param 	XTCLOCK - Clock source = XT 
* @param 	RCAUX - Clock fault --> upon reset goes to 10KHz (0x00)
*
* 
*/
static __INLINE void CLK_SelectClockSource( uint8_t source )
{
	if ( source == RC10KHZ ) *PMUCLK &= 0xFC;
	else if ( source == RC10MHZ ) {*PMUCLK = ( *PMUCLK & 0xFD ) | RC10MHZ;}
	else if ( source == XTCLOCK ) {*PMUCLK = ( *PMUCLK & 0xFE ) | XTCLOCK;}
	else { *PMUCLK |= RCAUX; }
}

// *********************
// PMU Related Registers
// *********************

#define PMURST   ((__IO uint8_t *) (PMU_BASE + 0x01))	//Power Management - Reset
#define	HWRST	0x80	// HW reset control, like POR
#define	SWRST	0x40	// SW reset control, does not affect clock and power, 
						// good for RF, ADC, ....
#define	SLEEPON	  0x20	// Deep Sleep Mode ON
#define	BOUTON	  0x02	// Brown out happening
#define	BOUTOFF	  0x00	// Brown out not happening
#define	PORCLR	  0x01	// POR Flag reset bit (Negative logic)

/**
* @brief Power Management Unit HW reset
*
* @param none
*
* 
*/
static __INLINE void PMU_HwReset( void )
{
	*PMURST |= HWRST;
}

/**
* @brief Power Management Unit sleep
*
* @param none
*
* 
*/
static __INLINE void PMU_Sleep( void )
{
	*PMURST |= SLEEPON;
}

/**
* @brief Power Management Unit SW reset
*
* @param none
*
* 
*/
static __INLINE void PMU_SwReset( void )
{
	*PMURST |= SWRST;
}


/**
* @brief Power Management Brownout
*
* @param none
* @return BOUTON or BOUTOFF
* 
*/
static __INLINE uint8_t PMU_CheckBrownout( void )
{
	return (*PMURST | BOUTON );
}


/**
* @brief Power Management POR Flag clear
*
* @param none
* 
*/
static __INLINE void PMU_ClearPorFlag( void )
{
	*PMURST &= ~PORCLR;
}

#define PMUBOR  ((__IO uint8_t *) (PMU_BASE + 0x02))	//Power Management - BOR
#define	BOREN	0x00	// Brown out enable
#define	BORDIS	0x80
#define	BORRSTEN	0x00	// BOR reset ON
#define	BORRSTDIS	0x08
#define	BORINTEN	0x04	// BOR interrupt ON
#define	BORINTDIS	0x00
#define	BOR20V		0x00	// BOR Level
#define	BOR22V		0x01
#define	BOR24V		0x02
#define	BOR26V		0x03

/**
* @brief BOR Control
*
* @param mode:
* @param BOREN - BOR enabled
* @param BORDIS	- BOR Disabled
* 
*/
static __INLINE void BOR_Control( uint8_t mode )
{
	if ( mode == BORDIS ) *PMUBOR |= BORDIS;
	else *PMUBOR &= (~BORDIS);
}

/**
* @brief BOR Reset Control
*
* @param mode:
* @param BORRSTEN - BOR reset enable
* @param BORDRSTDIS	- BOR reset disable
* 
*/
static __INLINE void BOR_ResetControl( uint8_t mode )
{
	if ( mode == BORRSTDIS ) *PMUBOR |= BORRSTDIS;
	else *PMUBOR &= (~BORRSTDIS);
}


/**
* @brief BOR Interrupt Control
*
* @param mode:
* @param BORINTEN - BOR interrupt enable
* @param BORINTDIS	- BOR interrupt disable
* 
*/
static __INLINE void BOR_IntControl( uint8_t mode )
{
	if ( mode == BORINTEN ) *PMUBOR |= BORINTEN;
	else *PMUBOR &= ~BORINTEN;
}


/**
* @brief BOR Level
*
* @param level:
* @param BOR20V - 2.0V
* @param BOR22V - 2.2V
* @param BOR24V - 2.4V
* @param BOR26V - 2.6V
* 
*/
static __INLINE void BOR_Level( uint8_t level )
{
	*PMUBOR = ( *PMUBOR & 0xFC ) | level;
}

#define PMUVREG  ((__IO uint8_t *) (PMU_BASE + 0x03))	//Power Management - BOR
#define LATCHUP_PROTECTION_ENABLED  0x1F
#define	LATCHUP_PROTECTION_DISABLED 0x00

/**
* @brief Latch Up Protection Control
*
* @param mode: LATCHUP_PROTECTION_ENABLED or LATCHUP_PROTECTION_DISABLED
* 
*/
static __INLINE void PMU_LAtchUp( uint8_t mode )
{
	*PMUVREG = ( *PMUVREG & 0xE0 ) | mode;
}

static __INLINE void VDD5V_EN( void )
{
	*PMUVREG &= (~0x80) ;
}

static __INLINE void VDD5V_DIS( void )
{
	*PMUVREG |= 0x80 ;
}

#define WKPTIME  ((__IO uint8_t *) (PMU_BASE + 0x04))	//Wake Up timer controller

/**
* @brief Wake Up Timing Control
*
* @param mode: mantissa and exponent
* T = Tclk*MANT*2^EXP
* Note: The range of the exponent is [0...12]
*/
static __INLINE void WKP_Timing( uint8_t mantissa, uint8_t exponent )
{
	*WKPTIME = (mantissa<<4) + exponent;
}

#define PMUMISC  ((__IO uint8_t *) (PMU_BASE + 0x05))	//PMU Misc.
#define	SERCLK_SYSTEM		0x80	//All serial interfaces run at 
									//system clock regardless of CKD[1...0]
#define	SERCLK_CKD			0x00	//All serial interfaces run at 
									//system clock defined by CKD[1...0]


// ******************************************
// PWM Related Registers
// ******************************************

#define PWM1_CONTROL ((__IO uint32_t *) (PWM_BASE + 0x00))	//

#define PWM1CTRL ((__IO uint8_t *) (PWM_BASE + 0x00))	//
#define PWMEN         0x80		//PWM1 enable
#define PWMDIS        0x00
#define PWMINV        0x10		//PWM1 inverted
#define PWMNINV       0x00
#define	PWM_SS_LONG   0x08		//SS_TIME = if 0 then counts 544 crystal 
#define	PWM_SS_SHORT  0x00		//oscillator clock cycles for short circuit 
								//test time.  If 1 then counts 800 clock cycles.
#define PRESC0        0x00    	//Prescaler: 2^0, 2^1, ....2^18
#define PRESC1        0x01
#define PRESC2        0x02
#define PRESC3        0x03
#define PRESC5        0x04
#define PRESC8        0x05
#define PRESC13       0x06
#define PRESC18       0x07

#define PWM1PER ((__IO uint8_t *) (PWM_BASE + 0x01))	//
#define PWM1PW  ((__IO uint8_t *) (PWM_BASE + 0x02))	//
#define PWM1EXT ((__IO uint8_t *) (PWM_BASE + 0x03))	//

#define PWM2_CONTROL ((__IO uint32_t *) (PWM_BASE + 0x04))	//

#define PWM2CTRL ((__IO uint8_t *) (PWM_BASE + 0x04))	//
#define PWMEN         0x80
#define PWMDIS        0x00
#define PWMINV        0x10
#define PWMNINV       0x00
#define	PWM_SS_LONG   0x08		//SS_TIME = if 0 then counts 544 crystal 
#define	PWM_SS_SHORT  0x00		//oscillator clock cycles for short circuit 
                                //test time.  If 1 then counts 800 clock cycles.
#define PRESC0        0x00    	//2^0, 2^1, 2^2, 2^3, 2^5, 2^8, 2^13, 2^18
#define PRESC1        0x01
#define PRESC2        0x02
#define PRESC3        0x03
#define PRESC5        0x04
#define PRESC8        0x05
#define PRESC13       0x06
#define PRESC18       0x07

#define PWM2PER ((__IO uint8_t *) (PWM_BASE + 0x05))	//
#define PWM2PW  ((__IO uint8_t *) (PWM_BASE + 0x06))	//
#define PWM2EXT ((__IO uint8_t *) (PWM_BASE + 0x07))	//

#define PWM1  0x01
#define PWM2  0x02

/**
* @brief PWM Setup
* @param number: PWM1 or PWM2
* @param en: PWMEN or PWMDIS
* @param inv : PWMINV or PWMNINV
* @param pres: Prescaler from PRESC0 = 2^0, PRESC1 = 2^1, PRESC2, PRESC3, PRESC5, 
*			   PRESC8, PRESC13, PRESC18
* @return : none 
*/
static __INLINE void PWM_Setup( uint8_t number, uint8_t en, uint8_t inv, uint8_t pres )
{
  if ( number == PWM1 )
  {
    *PWM1CTRL = en | inv | pres;
  }
  else if ( number == PWM2 )
  {
    *PWM2CTRL = en | inv | pres;
  }
}

/**
* @brief PWM Period setup
* @param number: PWM1 or PWM2
* @param per: Period [11...0]
* @return : none 
*/
static __INLINE void PWM_Period ( uint8_t number, uint16_t per )
{
  if ( number == PWM1 )
  {
    *PWM1PER = (uint8_t)(per >> 4);
    *PWM1EXT = (*PWM1EXT & 0x0F) | (((uint8_t) (( per << 4) & 0x00F0)));
  }
  else if ( number == PWM2 )
  {
    *PWM2PER = (uint8_t)(per >> 4);
    *PWM2EXT = (*PWM2EXT & 0x0F) | (((uint8_t) (( per << 4) & 0x00F0)));
  }
}


/**
* @brief PWM width setup
* @param number: PWM1 or PWM2
* @param width: [11...0]
* @return : none 
*/
static __INLINE void PWM_Width ( uint8_t number, uint16_t width )
{
  if ( number == PWM1 )
  {
    *PWM1PW = (uint8_t)(width >> 4);
    *PWM1EXT = (*PWM1EXT & 0xF0) | ((uint8_t)(width & 0x000F));   
  }
  else if ( number == PWM2 )
  {
    *PWM2PW = (uint8_t)(width >> 4);
    *PWM2EXT = (*PWM2EXT & 0xF0) | ((uint8_t)( width & 0x000F));       
  }
}

// *******************************************************
// RF Related Registers 
// *******************************************************

#define RXDATA  ((RxData *) (RF_BASE + 0x00))	//RF Data received
#define RXNUMBER  ((__IO uint8_t *) (RF_BASE + 0x0A))	//RF data (received) size in bits
// [6...0]
#define RFSTATUS  ((__IO uint8_t *) (RF_BASE + 0x0B))	//RF status
#define	RFEN	0x80	//RF is ON 
#define RFDIS   0x00    //
#define	SNIFEN	0x40	//Sniffer enable
#define	SNIFDIS	0x00	//Sniffer disabled, continuous mode on
#define	MSGRDY	0x20	//Define message ready: Reading returns one if m
			//a complete message is available and the receiver 
			//is in armed mode, otherwise returns zero.  
			//Write is ignored unless in armed mode, where 
			//writing a zero causes the receiver to leave 
			//the armed state and start decoding messages again.

#define	AGCOVF	0x10	//AGC overflow flag: Returns one if an overflow 
			//(signal too large) has occurred in the receiver.  
			//Intended for use in continuous mode, where the 
			//micro may want to reduce the gain setting. 

#define	SLICEROUT	0x02	//Slicer output
#define	RFSLEEP		0x01	//RF reception in sleep


/**
* @brief RF State (ON/OFF)
*
* @param none
* @return RFON or RFOFF
* 
*/
static __INLINE uint8_t RF_ReadState( void )
{
	return ( *RFSTATUS & RFEN );
}

/**
* @brief Set RF State (ON/OFF)
*
* @param none
* @return RFON or RFOFF
* 
*/
static __INLINE void RF_SetState( uint8_t state )
{
  if ( state == RFEN ) *RFSTATUS |= RFEN ;
  else  *RFSTATUS &= ~RFEN ;
}


/**
* @brief RF Sniffer Control (ON/OFF)
*
* @param sniffermode:
* @param SNIFEN or SNIFDIS
* 
*/
static __INLINE void RF_SnifferMode( uint8_t mode )
{
	if ( mode == SNIFEN )	{ *RFSTATUS |= SNIFEN; }
	else { *RFSTATUS &= ~SNIFEN; }
}



/**
* @brief RF Message Ready 
*
* @param none
* @return MSGRDY or 0
* 
*/
static __INLINE uint8_t RF_MessageReady( void )
{
	return ( *RFSTATUS & MSGRDY );
}


/**
* @brief RF ACG Overflow
* @param none
* @return AGCOVF or 0
* 
*/
static __INLINE uint8_t RF_AgcOverflow( void )
{
	return ( *RFSTATUS & AGCOVF );
}


/**
* @brief RF Reinit Decoder
*
* @param none
* 
*/
static __INLINE void RF_ReinitDecoder( void )
{
	*RFSTATUS &= ~MSGRDY; 
}

#define RFBITCTRL  ((__IO uint8_t *) (RF_SAP_BASE + 0x0000))	//RF Bit control
#define	LOWBPS		0x80	// Count bit times at half rate
#define	NORMALBPS	0x00	// NOrmal BPS
// [6...0] = Minimum number of bits for a message to be considered valid.

/**
* @brief RF Bit Timing rate control
*
* @param rate: Normal or half rate:
* @param LOWBPS or NORMALBPS
* 
*/
static __INLINE void RF_BitRateControl( uint8_t rate )
{
	if ( rate == LOWBPS )	{ *RFBITCTRL |= rate; }
	else { *RFBITCTRL &= ~LOWBPS; } 
}

/**
* @brief RF Bit Timing rate read
*
* @param none
* @return LOWBPS or NORMALBPS
* 
*/
static __INLINE uint8_t RF_ReadBitRateControl( void )
{
	return ( *RFBITCTRL & LOWBPS );
}

/**
* @brief Set RF Minimum number of bits.
*
* @param rate: 0 to 127
* 
*/
static __INLINE void RF_SetMinBitNumber( uint8_t numbits )
{
	*RFBITCTRL &= LOWBPS; 
	*RFBITCTRL |= numbits; 	
}

/**
* @brief Read RF Minimum number of bits.
*
* @param none.
* @return number of bits from 0 to 127
*/
static __INLINE uint8_t RF_ReadMinBitNumber( void )
{
	return ( *RFBITCTRL & 0x7F );	
}

#define RFAGCCTRL  ((__IO uint8_t *) (RF_SAP_BASE + 0x0001))	//RF AGC Control
#define	AGCEN	0x80	// AGC enabled or fixed gain (disabled)
#define	AGCDIS	0x00
// AGC_TRM [6...0]
//On read, returns the AGC bits being sent to the analog receiver.  On write, it 
//sets the values of a register which controls the behavior of the AGC system.  
//As such, typically read will return different value than had been written.
//The control register behavior depends on AGC_EN.  If AGC_EN is zero, then the 
//bits in the control register are used to directly control the AGC trim of the 
//analog blocks.  
//If AGC_EN is one, then the register controls several aspects of the AGC controller:
//AGC_EN[4] controls the step size of the AGC gain reduction.  
//If this bit is zero then the step size is 3dB per overflow, if one then the step 
//size is 6dB.
//AGC_EN[3:0] controls the default maximum gain, where the gain is notionally 
//3dB increased for each increase of this number.  Numbers up to 1010 are valid.  
//Numbers higher than 1010 will give unpredictable results.

								
/**
* @brief AGC control and trim
*
* @param mode: AGEN or AGDIS
* @param trim: [6...0] 
* 
*/
static __INLINE void RF_AgcControl( uint8_t mode, uint8_t trim )
{
  *RFAGCCTRL = mode | trim;
}



#define RFSLCCTRL  ((__IO uint8_t *) (RF_SAP_BASE + 0x0002))	//RF SLICER Control
#define ALPHA00   0x00        //Delay timer of the slicer 
#define ALPHA01   (0x01<<6)
#define ALPHA02   (0x02<<6)
#define ALPHA03   (0x03<<6)
#define BETA00    0x00        //Attack time of the slicer
#define BETA01    (0x01<<4)
#define BETA02    (0x02<<4)
#define BETA03    (0x03<<4)
#define FT18uS    0x00        //Time to allow slicer to fast bias
#define FT570uS   (0x01<<2)
#define FT860uS   (0x02<<2)
#define FT1140uS   (0x03<<2)
#define SDR00     0x00        //Symbol Decimation rate
#define SDR01     0x01
#define SDR02     0x02
#define SDR03     0x03

								
/**
* @brief Slicer control
*
* @param alpha:
* @param beta:
* @param ft:
* @param sdr:
* 
*/
static __INLINE void RF_SlicerControl( uint8_t alpha, uint8_t beta , uint8_t ft , uint8_t sdr )
{
  *RFSLCCTRL = alpha | beta | ft | sdr;
}



#define RFSYSTIME  ((__IO uint8_t *) (RF_SAP_BASE + 0x0003))	//RF System Time Control
#define	PLL32	(0x00<<6)		//Controls the time to wait for PLL to bias, measured in clock cycles of the Fxo/64:
#define	PLL48	(0x01<<6)		//00 => 32 cycles (570us for 3.58MHz crystal)
#define	PLL64	(0x02<<6)		//01 => 48 cycles (860us for 3.58 MHz crystal)
#define	PLL128	(0x03<<6)		//10 => 64 cycles (1.15ms for 3.58 MHz crystal)
					//11 => 128 cycles (2.29ms for 3.58MHz crystal)
#define	RFSLEEP4	(0x00<<3)	//Controls the time sleep between sniff cycles, measured in clock cycles of the fXO/64:
#define	RFSLEEP6	(0x01<<3)	//000 => 4*1024 cycles (73ms for 3.58MHz crystal)
#define	RFSLEEP8	(0x02<<3)	//001 => 6*1024 cycles (110ms for 3.58 MHz crystal)
#define	RFSLEEP10	(0x03<<3)	//010 => 8*1024 cycles (146ms for 3.58MHz crystal)
#define	RFSLEEP12	(0x04<<3)	//011 => 10*1024 cycles (183ms for 3.58MHz crystal)
#define	RFSLEEP14	(0x05<<3)	//100 => 12*1024 cycles (220ms for 3.58MHz crystal)
#define	RFSLEEP32	(0x06<<3)	//101 => 14*1024 cycles (256ms for 3.58 MHz crystal)
#define	RFSLEEP128	(0x07<<3)	//110 => 32*1024 cycles (586ms for 3.58MHz crystal)
								//111 => 128*1024 cycles (2.34s for 3.58MHz crystal)
#define	WAKET8		0x00		//Controls the time to stay awake after seeing a valid guardband, measured in clock cycles of the fXO/64:
#define	WAKET10		0x01		//000 => 8*1024 cycles (146ms for 3.58MHz crystal)
#define	WAKET12		0x02		//001 => 10*1024 cycles (183ms for 3.58 MHz crystal)
#define	WAKET14		0x03		//010 => 12*1024 cycles (220ms for 3.58MHz crystal)
#define	WAKET16		0x04		//011 => 14*1024 cycles (256ms for 3.58MHz crystal)
#define	WAKET18		0x05		//100 => 16*1024 cycles (293ms for 3.58MHz crystal)
#define	WAKET24		0x06		//101 => 18*1024 cycles (330ms for 3.58 MHz crystal)
#define	WAKET32		0x07		//110 => 24*1024 cycles (440ms for 3.58MHz crystal)
								//111 => 32*1024 cycles (586ms for 3.58MHz crystal)

								
/**
* @brief Set time to wait for PLL bias 
*
* @param btime:
* 
*/
static __INLINE void RF_SetPllBiasTime( uint8_t btime )
{
	*RFSYSTIME = ( *RFSYSTIME & 0x3F ) | btime;
}


/**
* @brief Read time to wait for PLL bias 
*
* @param none
* @return sniff sleep time. (PLLn)
*/
static __INLINE uint8_t RF_ReadPllBiasTime( void )
{
	return ( *RFSYSTIME & 0xC0 );
}


/**
* @brief Set sleep time between sniffs
*
* @param stime
*/
static __INLINE void RF_SetSleepTime( uint8_t stime )
{
	*RFSYSTIME = ( *RFSYSTIME & 0xC7 ) | stime;
}

/**
* @brief Read sleep time between sniffs
*
* @param none
* @return sniff sleep time. (RFSLEEPn)
*
*/
static __INLINE uint8_t RF_ReadSleepTime( void )
{
	return ( *RFSYSTIME & 0x38 );
}

/**
* @brief Set wake time in sniffs
*
* @param wtime
*/
static __INLINE void RF_SetWakeTime( uint8_t wtime )
{
	*RFSYSTIME = ( *RFSYSTIME & 0xF8 ) | wtime;
}

/**
* @brief Read wake time in sniffs
*
* @param wtime
* @return Wake Time 
*/
static __INLINE uint8_t RF_ReadWakeTime( void )
{
	return ( *RFSYSTIME & 0x07 );
}

#define RFDECTIME  ((__IO uint16_t *) (RF_SAP_BASE + 0x0004))	//RF Decoder time control
//Min_Te [15...12]
//Max_Te [11...6]
//Min_Gb [5...0]

/**
* @brief Set Min_Te timer
*
* @param minte - The value to be used to calculate the minimum time for a bit 
*				 to be accepted as valid: = (1+Min_Te)*(12*16)/3.58e6
*/
static __INLINE void RF_SetMinTe( uint8_t minte )
{
	*RFDECTIME = ( *RFDECTIME & 0x0FFF ) | ( minte << 12);
}

/**
* @brief Read Min_Te timer
*
* @param none
* @return min_te 
*/
static __INLINE uint8_t RF_ReadMinTe( void )
{
	return ( ( uint8_t ) ( *RFDECTIME >> 12 ) );
}

/**
* @brief Set Max_Te timer
*
* @param maxte - The value to be used to calculate the maximum time for a bit 
*				 to be accepted as valid: = (1 + Min_Te + Max_Te)*(12*16)/3.58e6
*/
static __INLINE void RF_SetMaxTe( uint8_t maxte )
{
	*RFDECTIME = ( *RFDECTIME & 0xF03F ) | ( maxte << 6 );
}

/**
* @brief Read Max_Te timer
*
* @param none
* @return max_te 
*/
static __INLINE uint8_t RF_ReadMaxTe( void )
{
	return ( (uint8_t) ( *RFDECTIME >> 6 ) );
}

/**
* @brief Set Min_Gb timer
*
* @param mingb - The value to be used to calculate the minimum time for a packet
*				 to be accepted as valid: = (1 + Min_Te + Max_Te + Min_Gb)*(12*16)/3.58e6
*/
static __INLINE void RF_SetMinGb( uint8_t mingb )
{
	*RFDECTIME = ( *RFDECTIME & 0xFFC0 ) | ( mingb );
}

/**
* @brief Read Min_Gb timer
*
* @param none
* @return mingb 
*/
static __INLINE uint8_t RF_ReadMinGb( void )
{
	return ( (uint8_t) ( *RFDECTIME & 0x003F) );
}

#define RFSNIFMODE  ((__IO uint8_t *) (RF_SAP_BASE + 0x0006))	//RF sniffer mode control
#define RTSUP      0x00        //Selects a source for real-time output:
#define RTDE      (0x01<<6)    //00 => supervisor clock
#define RTSLICER  (0x02<<6)    //01 => decimator output (serialized stream)
#define RTPLL     (0x03<<6)    //10 => slicer output
                               //11 => PLL_EN (high if analog blocks enabled)

#define RFONEN    (0x01<<5)
#define RFONDIS   (0x00<<6)
//snifferNE [3...0]


/**
* @brief Set Sniffer mode timing
*
* @param rt - source for real-time output:
* RTSUP      supervisor clock
* RTDE       decimator output (serialized stream)   
* RTSLICER   slicer output
* RTPLL      PLL_EN (high if analog blocks enabled)
* @param rfall - RF always on or not (For test) FRONDIS or RFONEN
* @param sniff - Number of edges to check in each sniff cycle before committing 
*                to a long wake cycle.  When in sniff mode, the first SNIFF_NE 
*                edges are tested for valid timing.  If any one of these first 
*                edges are badly timed, then the receiver will go to sleep.
*/
static __INLINE void RF_SetSnifRt( uint8_t rt, uint8_t rfall, uint8_t sniff )
{
  *RFSNIFMODE = rt | rfall | sniff ;
}


#define RFAGCMON  ((__IO uint8_t *) (RF_SAP_BASE + 0x0007))		//RF AGC monitor

/**
* @brief Read AGC gain value
*
* @param none
* @return agc
*/
static __INLINE uint8_t RF_ReadAgc( void )
{
	return ( *RFAGCMON );
}

#define RFSIGI  ((__IO uint16_t *) (RF_SAP_BASE + 0x0008))		//RF I channel DC cal. value
#define RFSIGQ  ((__IO uint16_t *) (RF_SAP_BASE + 0x000A))		//RF Q channel DC cal. value
#define RFPKDETHI  ((__IO uint16_t *) (RF_SAP_BASE + 0x000C))	//RF peak detector high
#define RFPKDETLO  ((__IO uint16_t *) (RF_SAP_BASE + 0x000E))	//RF peak detector low

// ****************************************************************************
// LED and GPIO RELATED REGISTERS 
// ****************************************************************************

#define PORTA ((__IO uint8_t *) (ASIC_7B_BASE + 0x60))	// PORTA I/O
#define PORTB ((__IO uint8_t *) (ASIC_7B_BASE + 0x61))	// PORTB I/O
#define PORTC ((__IO uint8_t *) (ASIC_7B_BASE + 0x62))	// PORTC I/O
#define PORTD ((__IO uint8_t *) (ASIC_7B_BASE + 0x63))	// PORTD I/O
#define PORTE ((__IO uint8_t *) (ASIC_7B_BASE + 0x64))	// PORTE I/O
#define	PORTDOE	((__IO uint8_t *) (ASIC_7B_BASE + 0x65))	// PORTD Output enable
#define	PDOE7_EN	0x80
#define	PDOE7_DIS	0x00
#define	PDOE6_EN	0x40
#define	PDOE6_DIS	0x00
#define	PDOE5_EN	0x20
#define	PDOE5_DIS	0x00
#define	PDOE4_EN	0x10
#define	PDOE4_DIS	0x00
#define	PDOE3_EN	0x08
#define	PDOE3_DIS	0x00
#define	PDOE2_EN	0x04
#define	PDOE2_DIS	0x00
#define	PDOE1_EN	0x02
#define	PDOE1_DIS	0x00
#define	PDOE0_EN	0x01
#define	PDOE0_DIS	0x00

#define	PORTEOE	((__IO uint8_t *) (ASIC_7B_BASE + 0x66))	// PORTE Output enable
#define	PEOE3_EN	0x80
#define	PEOE3_DIS	0x00
#define	PEOE2_EN	0x40
#define	PEOE2_DIS	0x00
#define	PEOE1_EN	0x20
#define	PEOE1_DIS	0x00
#define	PEOE0_EN	0x10
#define	PEOE0_DIS	0x00
#define MODESPI     0x08
#define MODEI2C     0x04
#define MODEUART    0x02
#define MODELIN     0x01



/**
* @brief PORTE output enable
*
* @param mode: 1 = Output enabled, 0 = Output Disabled
*/
static __INLINE void PE_Oe( uint8_t mode )
{
	*PORTEOE = ( *PORTEOE & 0x0F ) | ( mode << 4 );
}

#define	PCONF	((uint8_t *) (ASIC_7B_BASE + 0x67))	// Aux. config register
#define I2C_NO_PULLUP       0x00
#define I2C_1K_PULLUP       0x40
#define I2C_10K_PULLUP      0x80
#define I2C_100K_PULLUP     0xC0
#define	LINTX_EN			0x10	//LIN transmission enable
#define	LINTX_DIS			0x00
#define	LINSWAP_EN			0x08	//LIN pins swap: TX=PD[6] and RX = PD[7]
#define	LINSWAP_DIS			0x00	//LIN pins NOT swap: TX=PD[7] and RX = PD[6]
#define	UARTSWAP_EN			0x04	//UART pins swap: TX=PD[5] and RX = PD[4]
#define	UARTSWAP_DIS		0x00	//UART pins NOT swap: TX=PD[4] and RX = PD[5]
#define	LINPOL_NORMAL		0x00	//LIN polarity normal
#define	LINPOL_INVERTED		0x02	//LIN polarity inverted
#define	UARTPOL_NORMAL		0x00	//UART polarity normal
#define	UARTPOL_INVERTED	0x01	//UART polarity inverted

/**
* @brief UART Setup
*
* @param swap: UART pins swap: UARTSWAP_EN or UARTSWAP_DIS
* @param pol: UART signals polarity: UARTPOL_NORMAL or UARTPOL_INVERTED
*/
static __INLINE void UART_Setup( uint8_t swap,  uint8_t pol )
{
	*PCONF = (*PCONF & 0xFA) | swap | pol;
}

/**
* @brief LIN Setup
*
* @param txen: LIN Transmission Enabled: LINTX_EN or LINTX_DIS
* @param swap: LIN pins swap: LINSWAP_EN or LINSWAP_DIS
* @param pol: LIN signals polarity: LINPOL_NORMAL or LINPOL_INVERTED
*/
static __INLINE void LIN_Setup( uint8_t txen,  uint8_t swap,  uint8_t pol )
{
	*PCONF = (*PCONF & 0xE5) | txen | swap | pol;
}

// Structure of configuration pins:
// [5] = imask, [4] = oe, [3] = st, [2] = pu, [1] = pd, [0] = level


#define	PA0CFG	((uint8_t *) (ASIC_7B_BASE + 0x68))	// PORTA.0 Config register
#define	PA1CFG	((uint8_t *) (ASIC_7B_BASE + 0x69))	// PORTA.1 Config register
#define	PA2CFG	((uint8_t *) (ASIC_7B_BASE + 0x6A))	// PORTA.2 Config register
#define	PA3CFG	((uint8_t *) (ASIC_7B_BASE + 0x6B))	// PORTA.3 Config register
#define	PA4CFG	((uint8_t *) (ASIC_7B_BASE + 0x6C))	// PORTA.4 Config register
#define PA4_PWM2_SNS_EN		0x10	//Enable PWM2 short circuit sensor in PA4

#define	PA5CFG	((uint8_t *) (ASIC_7B_BASE + 0x6D))	// PORTA.5 Config register
#define PA5_PWM1_SNS0_EN	0x20	//Enable PWM1 short circuit sensor0 in PA5

#define	PA6CFG	((uint8_t *) (ASIC_7B_BASE + 0x6E))	// PORTA.6 Config register
#define PA6_PWM1_SNS1_EN	0x40	//Enable PWM1 short circuit sensor1 in PA6

#define	PA7CFG	((uint8_t *) (ASIC_7B_BASE + 0x6F))	// PORTA.7 Config register
#define PA7_PWM2		0x80	//Enable PWM2 output in PA7

#define	PB0CFG	((uint8_t *) (ASIC_7B_BASE + 0x70))	// PORTB.0 Config register
#define PB0_PWM1		0x01	//Enable PWM1 output in PB0
#define	PB1CFG	((uint8_t *) (ASIC_7B_BASE + 0x71))	// PORTB.1 Config register
#define PB1_PWM2		0x02	//Enable PWM2 output in PB1
#define	PB1_ACTL		0x04	//Active Level for the output (P-Mos = 0, N-Mos = 1)

#define	PB2CFG	((uint8_t *) (ASIC_7B_BASE + 0x72))	// PORTB.2 Config register
#define	PB3CFG	((uint8_t *) (ASIC_7B_BASE + 0x73))	// PORTB.3 Config register
#define	PB4CFG	((uint8_t *) (ASIC_7B_BASE + 0x74))	// PORTB.4 Config register
#define	PB5CFG	((uint8_t *) (ASIC_7B_BASE + 0x75))	// PORTB.5 Config register
#define	PB6CFG	((uint8_t *) (ASIC_7B_BASE + 0x76))	// PORTB.6 Config register
#define	PB7CFG	((uint8_t *) (ASIC_7B_BASE + 0x77))	// PORTB.7 Config register


#define	PC0CFG	((uint8_t *) (ASIC_7B_BASE + 0x78))	// PORTC.0 Config register
#define	PC1CFG	((uint8_t *) (ASIC_7B_BASE + 0x79))	// PORTC.1 Config register
#define	PC2CFG	((uint8_t *) (ASIC_7B_BASE + 0x7A))	// PORTC.2 Config register
#define	PC3CFG	((uint8_t *) (ASIC_7B_BASE + 0x7B))	// PORTC.3 Config register
#define PC3_PWM2		0x08	//Enable PWM2 output in PC3

#define	PC4CFG	((uint8_t *) (ASIC_7B_BASE + 0x7C))	// PORTC.4 Config register
#define	PC5CFG	((uint8_t *) (ASIC_7B_BASE + 0x7D))	// PORTC.5 Config register
#define PC5_PWM1		0x20	//Enable PWM1 output in PC5

#define	PC6CFG	((uint8_t *) (ASIC_7B_BASE + 0x7E))	// PORTC.6 Config register
#define	PC7CFG	((uint8_t *) (ASIC_7B_BASE + 0x7F))	// PORTC.7 Config register
#define PC7_PWM1		0x40	//Enable PWM1 output in PC7
#define	PC7_ACTL		0x80	//Active Level for the output (P-Mos = 0, N-Mos = 1)

#define	SSLOW	0x00		// Short Circuit Levels
#define	SSHIGH 	0x01
#define	PTA		0x00		// Port to be configured
#define	PTB		0x01
#define	PTC		0x02
#define	PTD		0x03
#define	PTE		0x04

	

/**
* @brief Port Configuration 
*
* @param pt - port to be configured: PTA, PTB, PTC, PTD, PTE
* @param ss - auxiliar function enable. Must use defines:
*   @param PORTA --> PA4_PWM2_SNS_EN, PA5_PWM1_SNS0_EN, PA6_PWM1_SNS1_EN, PA7_PWM2
*   @param PORTB --> PB0_PWM1, PB1_PWM2, PB1_ACTL
*   @param PORTC --> PC3_PWM2, PC5_PWM1, PC7_PWM1 ,PC7_ACTL
* @param imask - Interrupt enable mask, active High
* @param oe - output enable, active high
* @param st - Current capability (Strength) of pullup/down (0=100uA, 1=5mA)
* @param pu - pull-up enable, active high
* @param pd - pull-down enable, active high
* @param level - input threshold level. (0 = 1.65V, 1 = 4V)
*/
static __INLINE void Port_Config( uint8_t pt, uint8_t ss, uint8_t imask, uint8_t oe, uint8_t st, uint8_t pu, uint8_t pd, uint8_t level )
{
	if ( pt == PTD )
	{
		*PORTDOE = oe;
	}
	else if ( pt == PTE )
	{
		PE_Oe( oe );
	}
	else if ( pt == PTA )  // PIN support: PA4~7, PB1~PB3 and PC4~PC7
	{
		*PA0CFG = 			  ( ( oe & 0x01) << 4 ) | ( ( st & 0x01) << 3 ) | ( ( pu & 0x01) << 2 ) | ( ( pd & 0x01) << 1 ) | ( ( level & 0x01) << 0 );
		*PA1CFG = 			  ( ( oe & 0x02) << 3 ) | ( ( st & 0x02) << 2 ) | ( ( pu & 0x02) << 1 ) | ( ( pd & 0x02) >> 0 ) | ( ( level & 0x02) >> 1 );	
		*PA2CFG = 			  ( ( oe & 0x04) << 2 ) | ( ( st & 0x04) << 1 ) | ( ( pu & 0x04) << 0 ) | ( ( pd & 0x04) >> 1 ) | ( ( level & 0x04) >> 2 );		
		*PA3CFG = 			  ( ( oe & 0x08) << 1 ) | ( ( st & 0x08) << 0 ) | ( ( pu & 0x08) >> 1 ) | ( ( pd & 0x08) >> 2 ) | ( ( level & 0x08) >> 3 );		
		*PA4CFG = ( ( ss & 0x10) << 2 ) | ( ( imask & 0x10) << 1 ) | ( ( oe & 0x10) << 0 ) | ( ( st & 0x10) >> 1 ) | ( ( pu & 0x10) >> 2 ) | ( ( pd & 0x10) >> 3 ) | ( ( level & 0x10) >> 4 );	
		*PA5CFG = ( ( ss & 0x20) << 1 ) | ( ( imask & 0x20) << 0 ) | ( ( oe & 0x20) >> 1 ) | ( ( st & 0x20) >> 2 ) | ( ( pu & 0x20) >> 3 ) | ( ( pd & 0x20) >> 4 ) | ( ( level & 0x20) >> 5 );	
		*PA6CFG = ( ( ss & 0x40) << 0 ) | ( ( imask & 0x40) >> 1 ) | ( ( oe & 0x40) >> 2 ) | ( ( st & 0x40) >> 3 ) | ( ( pu & 0x40) >> 4 ) | ( ( pd & 0x40) >> 5 ) | ( ( level & 0x40) >> 6 );	
		*PA7CFG = ( ( ss & 0x80) >> 1 ) | ( ( imask & 0x80) >> 2 ) | ( ( oe & 0x80) >> 3 ) | ( ( st & 0x80) >> 4 ) | ( ( pu & 0x80) >> 5 ) | ( ( pd & 0x80) >> 6 ) | ( ( level & 0x80) >> 7 );	
	}
	else if ( pt == PTB ) 
	{
		*PB0CFG = ( ( ss & 0x01) << 6 )                            | ( ( oe & 0x01) << 4 ) | ( ( st & 0x01) << 3 ) | ( ( pu & 0x01) << 2 ) | ( ( pd & 0x01) << 1 ) | ( ( level & 0x01) << 0 );
		*PB1CFG = ( ( ss & 0x02) << 5 ) | ( ( imask & 0x02) << 4 ) | ( ( oe & 0x02) << 3 ) | ( ( st & 0x02) << 2 ) | ( ( pu & 0x02) << 1 ) | ( ( pd & 0x02) >> 0 ) | ( ( level & 0x02) >> 1 );	
		*PB2CFG =  			  ( ( imask & 0x04) << 3 ) | ( ( oe & 0x04) << 2 ) | ( ( st & 0x04) << 1 ) | ( ( pu & 0x04) << 0 ) | ( ( pd & 0x04) >> 1 ) | ( ( level & 0x04) >> 2 );		
		*PB3CFG =  			  ( ( imask & 0x08) << 2 ) | ( ( oe & 0x08) << 1 ) | ( ( st & 0x08) << 0 ) | ( ( pu & 0x08) >> 1 ) | ( ( pd & 0x08) >> 2 ) | ( ( level & 0x08) >> 3 );		
		*PB4CFG =  			  ( ( imask & 0x10) << 1 ) | ( ( oe & 0x10) << 0 ) | ( ( st & 0x10) >> 1 ) | ( ( pu & 0x10) >> 2 ) | ( ( pd & 0x10) >> 3 ) | ( ( level & 0x10) >> 4 );	
		*PB5CFG =  						     ( ( oe & 0x20) >> 1 ) | ( ( st & 0x20) >> 2 ) | ( ( pu & 0x20) >> 3 ) | ( ( pd & 0x20) >> 4 ) | ( ( level & 0x20) >> 5 );	
		*PB6CFG =  						     ( ( oe & 0x40) >> 2 ) | ( ( st & 0x40) >> 3 ) | ( ( pu & 0x40) >> 4 ) | ( ( pd & 0x40) >> 5 ) | ( ( level & 0x40) >> 6 );	
		*PB7CFG =  						     ( ( oe & 0x80) >> 3 ) | ( ( st & 0x80) >> 4 ) | ( ( pu & 0x80) >> 5 ) | ( ( pd & 0x80) >> 6 ) | ( ( level & 0x80) >> 7 );		
	}
	else if ( pt == PTC )
	{
		*PC0CFG =			                             ( ( oe & 0x01) << 4 ) | ( ( st & 0x01) << 3 ) | ( ( pu & 0x01) << 2 ) | ( ( pd & 0x01) << 1 ) | ( ( level & 0x01) << 0 );
		*PC1CFG =			                             ( ( oe & 0x02) << 3 ) | ( ( st & 0x02) << 2 ) | ( ( pu & 0x02) << 1 ) | ( ( pd & 0x02) >> 0 ) | ( ( level & 0x02) >> 1 );	
		*PC2CFG =  			                             ( ( oe & 0x04) << 2 ) | ( ( st & 0x04) << 1 ) | ( ( pu & 0x04) << 0 ) | ( ( pd & 0x04) >> 1 ) | ( ( level & 0x04) >> 2 );		
		*PC3CFG = ( ( ss & 0x08) << 3 )                            | ( ( oe & 0x08) << 1 ) | ( ( st & 0x08) << 0 ) | ( ( pu & 0x08) >> 1 ) | ( ( pd & 0x08) >> 2 ) | ( ( level & 0x08) >> 3 );		
		*PC4CFG = 			  ( ( imask & 0x10) << 1 ) | ( ( oe & 0x10) << 0 ) | ( ( st & 0x10) >> 1 ) | ( ( pu & 0x10) >> 2 ) | ( ( pd & 0x10) >> 3 ) | ( ( level & 0x10) >> 4 );	
		*PC5CFG = ( ( ss & 0x20) << 1 ) | ( ( imask & 0x20) << 0 ) | ( ( oe & 0x20) >> 1 ) | ( ( st & 0x20) >> 2 ) | ( ( pu & 0x20) >> 3 ) | ( ( pd & 0x20) >> 4 ) | ( ( level & 0x20) >> 5 );	
		*PC6CFG = 			  ( ( imask & 0x40) >> 1 ) | ( ( oe & 0x40) >> 2 ) | ( ( st & 0x40) >> 3 ) | ( ( pu & 0x40) >> 4 ) | ( ( pd & 0x40) >> 5 ) | ( ( level & 0x40) >> 6 );	
		*PC7CFG = ( ( ss & 0x80) >> 0 ) | ( ( imask & 0x80) >> 2 ) | ( ( oe & 0x80) >> 3 ) | ( ( st & 0x80) >> 4 ) | ( ( pu & 0x80) >> 5 ) | ( ( pd & 0x80) >> 6 ) | ( ( level & 0x80) >> 7 );		
	}
}
/* //Code Example: Config Port C, PWM2 in PC3, PWM1 in PC7 with N-MOS transistor (Active level = 1 = High),
   //              Interrupt mask in PC0 and PC1, output enable for PC3, PC5 and PC7, low current in pull-up/down, 
   //			   pull-up in PC0 and PC1, no pull downs, input threshold high for PC0 and PC1
	Port_Config( PTC, (PC3_PWM2 | PC7_PWM1 | PC7_ACTL), 0x03, 0xA8, 0x00, 0x03, 0x00, 0x03 );
*/

#define LEDIO ((uint8_t *) (ASIC_7B_BASE + 0x59))	// LED I/O (PORTB.4)
#define	LEDEN	0x20	// Enable LED mode for the pin
#define	LEDDIS	0x00	// Disable LED, GPIO enabled for the pin
// Led Current [4...0]	// LED current selection -->  I = 4mA*(value) + nominal
// Nominal = 2mA

/**
* @brief LED Control
*
* @param mode: LEDEN or LEDDIS (Enabled or Disabled)
*/
static __INLINE void LED_Control( uint8_t mode )
{
  
  if ( mode == LEDEN ) { *LEDIO |= LEDEN; *PORTB |= 0x01; }       //Activate PB0 as LED output
  else *LEDIO &= ~LEDEN;
}


/**
* @brief LED Current Control
*
* @param curr: From 2 to 64 (mA)
* I = 4mA*(value) + nominal
*/
static __INLINE void LED_Current( uint8_t curr )
{
  *LEDIO = ( *LEDIO & 0xE1 ) | ( ((curr-2)/4) << 1 );
}

#define RFNX ((uint8_t *) (TRIM_SAP_BASE + 0x00))	// RF NX Bits
#define RFNF ((uint8_t *) (TRIM_SAP_BASE + 0x01))	// RF NF Bits
#define RFCTRL0 ((uint8_t *) (TRIM_SAP_BASE + 0x02))	// RF Control 0
#define	HISDEN	0x80		//Controls image reject mixer to control whether
							//to use high-side or low side mixing.
#define	HISDDIS	0x00		//
#define	FEN		0x40		//If one then the PLL is used in fractional-N mode,
							//otherwise it is integer-N
#define	FDIS	0x00		//
#define	CPT00	0x00		//Charge pump current trim (settling time)
#define	CPT01	(0x01<<4)	//
#define	CPT02	(0x02<<4)	//
#define	CPT03	(0x03<<4)	//
// [3...0] Loop filter resistor trim (overshoot)

/**
* @brief RF Control 0
*
* @param rejection: Single side band demodulation with Weaver image
* @param rejection architecture --> HISDEN = 1  = high side band; HISDDIS = 0 = Low side band I
* @param fractional: FEN = fractional active, FDIS = integer active
* @param cpt - charge pump trim: CPT00 to CPT3
* @param lfrt - 0...15
*/
static __INLINE void RF_Setup0( uint8_t rejection, uint8_t fractional, uint8_t cpt, uint8_t lfrt )
{
	*RFCTRL0 = rejection | fractional | cpt | lfrt;
}

#define RFCTRL1 ((uint8_t *) (TRIM_SAP_BASE + 0x03))	// RF Control 1
#define	DIV_LO2_16	(0x00<<6)		//Controls the divide value of second LO. Divide value =
#define	DIV_LO2_17	(0x01<<6)		//16 +DIV_LO2[1:0]
#define	DIV_LO2_18	(0x02<<6)	
#define	DIV_LO2_19	(0x03<<6)	
#define	BBGAIN0dB	(0x00<<4)		//Base band gain 00 = 0dB; 01 = 6dB; 10 = 12dB; 11 =
#define	BBGAIN6dB	(0x01<<4)		//18dB
#define	BBGAIN12dB	(0x02<<4)	
#define	BBGAIN18dB	(0x03<<4)	
#define	LNADR1K		(0x01<<3)		//LNA drain resistor 0 = 2Kohms, 1 = 1Kohm
#define	LNADR2K 	(0x00<<3)		//
#define	LNABIAS0uA	(0x00)			//LNA bias current. 000 = 0; 001 = 200uA; 010 = 500uA;
#define	LNABIAS200uA	(0x01)		//011 = 700uA; 100 = 1mA; 101 = 1.2mA; 110 = 1.5mA
#define	LNABIAS500uA	(0x02)		//111 = 1.7mA
#define	LNABIAS700uA	(0x03)		
#define	LNABIAS1mA	(0x04)		
#define	LNABIAS1p2mA	(0x05)		
#define	LNABIAS1p5mA	(0x06)
#define	LNABIAS1p7mA	(0x07)

/**
* @brief RF Control 1
*
* @param lodiv: DIV_LO2_16 to DIV_LO2_19
* @param bbg: BBGAIN0dB to BBGAIN18dB
* @param lnadr: LNADREN or LNADRDIS
* @param lnab - LNABIAS0uA to LNABIAS1p7mA
*/
static __INLINE void RF_Setup1( uint8_t lodiv, uint8_t bbg, uint8_t lnadr, uint8_t lnab )
{
	*RFCTRL1 = lodiv | bbg | lnadr | lnab;
}

// ****************************************************************************
// I2C RELATED REGISTERS 
// ****************************************************************************

#define I2CSTATUS   ((volatile uint8_t *) (I2C_BASE + 0x00))	//All bits read-only!
#define	ACKRCVD		0x00	//ACK Received
#define	NACKRCVD	0x80	//NACK Received
#define	ADDRCVD		0x40	//ADDRESS Received (Slave Mode)
#define	DATRCVD		0x00	//DATA Received (Slave Mode)
#define	STRRCVD		0x20	//Set when START-Bit received, cleared upon reading
#define	STPRCVD		0x10	//Set when STOP-Bit Received, cleared upon reading
#define	RWBUSY		0x08	//Master Mode: 1 = I2C bus being accessed, otherwise idle
                            //Slave Mode: 	1 = I2C read operation (Slave transmits data) 
                            //              0 = I2C write operation (Slave receives data)
#define	BUFFUL		0x04	//Buffer Full, either because it received data or there is 
                            //one byte to be transmitted. Cleared when buffer is empty
#define	BUFINVLD	0x02	//Set when the internal shift register is full and I2CDATA is
                            //writen to. Cleared when this buffer is empty.
#define	BUFOVFL		0x01	//Set if the internal shift register is full and another
                            //byte is received from I2C bus. Cleared when the register
                            //is read. While set the shift-in of bits from bus is stopped.


/**
* @brief Read ACK/NACK from I2C Interface
*
* @param none
* @return ACK = 1
* @return NACK = 0
*
* 
*/
static __INLINE uint8_t I2C_Read_ACK ( void )
{
	return ((*I2CSTATUS & NACKRCVD) ? FALSE : TRUE);
}

/**
* @brief Check if received Address or Data from I2C Interface
*
* @param none
* @return ADDRESS = 0x40
* @return DATA = 0x00
*
* 
*/
static __INLINE uint8_t I2C_Received_Address_Data ( void )
{
	return ( *I2CSTATUS & ADDRCVD );
}


/**
* @brief Check if received Start Bit from I2C Interface
*
* @param none
* @return START BIT = 0x20
* @return No Start Bit = 0x00
*
* 
*/
static __INLINE uint8_t I2C_Received_Start ( void )
{
	return ( *I2CSTATUS & STRRCVD );
}


/**
* @brief Check if received Stop Bit from I2C Interface
*
* @param none
* @return STOP BIT = 0x10
* @return No Stop Bit = 0x00
*
* 
*/
static __INLINE uint8_t I2C_Received_Stop ( void )
{
	return ( *I2CSTATUS & STPRCVD );
}


/**
* @brief Check if bus idle (Master Mode)
*
* @param none
* @return IDLE = 0x01
* @return Busy = 0x00 (Bus being accessed)
*
* 
*/
static __INLINE uint8_t I2C_Master_Bus_Idle ( void )
{
	return ( ( *I2CSTATUS & RWBUSY ) ? 0x00: 0x01 );
}



/**
* @brief Check if Slave is transmitting data (Read Operation) or 
* @brief receiving data (Write Operation)
*
* @param none
* @return 0x01 = Slave Transmitting Data (I2C Read Data)
* @return 0x00 = Slave Receiving Data (I2C Write Data)
*
* 
*/
static __INLINE uint8_t I2C_Slave_Transmitting ( void )
{
	return ( *I2CSTATUS & RWBUSY );
}



/**
* @brief I2C Check if Buffer is full.
*
* @param none
* @return 0x04 = Buffer Full
* @return 0x00 = Buffer Empty
*
* When set, the I2C internal shift register is full by receiving data/address
* byte from I2C bus (incoming data) or I2CDATA register (outgoing data). 
* Cleared when the buffer is empty.
*/
static __INLINE uint8_t I2C_Buffer_Full ( void )
{
	return ( *I2CSTATUS & BUFFUL );
}


/**
* @brief Check if Transmit Buffer is Overloaded.
*
* @param none
* @return 0x02 = Buffer Overloaded
* @return 0x00 = Buffer Empty
*
* If the I2C internal shift register is full and I2CDATA register is written
* then this bit is set. Cleared when this register is read. The I2CDATA 
* register write does not go through.
*/
static __INLINE uint8_t I2C_Transmit_Buffer_Overloaded ( void )
{
	return ( *I2CSTATUS & BUFINVLD );
}


/**
* @brief Check if Reception Buffer is Overloaded.
*
* @param none
* @return 0x01 = Buffer Overloaded
* @return 0x00 = Buffer Empty
*
* If the AIM internal shift register is full and I2CDATA register is written
* then this bit is set. Cleared when this register is read. The I2CDATA 
* register write does not go through.
*/
static __INLINE uint8_t I2C_Receive_Buffer_Overloaded ( void )
{
	return ( *I2CSTATUS & BUFOVFL );
}

#define I2CCTRL1    ((volatile uint8_t *) (I2C_BASE + 0x01))
#define	RSBEN				0x80	//Repeated start-bit transmit enable, cleared by HW. (Master Only)
									//(when set the I2C transmits Repeated Start bit)
#define	CLK_STRETCH_ON		0x00	//Clock Stretch: 0 = SCL held low, 1 = released (Slave Only)
#define	CLK_STRETCH_OFF		0x40	//
#define	GC_ON				0x20	//General call address enable. (Slave Only)
#define	GC_OFF				0x00	//General call address disabled. (Slave Only)
#define	RCVSTRT				0x10	//Receive start bit, cleared by HW.(Master Only)
									//(The receive operation starts when this bit is set)
#define	SET_NACK			0x08	//NACK value to be sent upon reception of byte through I2C (Master Only)
#define	SET_ACK				0x00	//ACK value to be sent upon reception of byte through I2C (Master Only)
#define	ACKEN				0x04	//ACK send bit. (When set I2C transmits the content of bit3 (SET_ACK or SET_NACK) as the 
									//Acknowledge/Not Acknowledge bit. (Master Only)
#define	STPSIZE				0x02	//Master Mode: When set sends the stop bit. (Cleared by HW)
#define	_10_BIT_ADDRESS		0x02	//Slave Mode: 	1 = 10-bit address
#define	_7_BIT_ADDRESS		0x00	//				0 = 7-bit address
#define	STRSTRE				0x01	//Master Mode: 	When set sends the start bit (Cleared by HW)	
									//Slave Mode: 	1 = Clock streched enabled before receiving next byte.


/**
* @brief I2C Master issue a repeated Start-bit.
*
* @param none
* @return none
*
* 
*/
static __INLINE void I2C_Master_Repeat_Start ( void )
{
	*I2CCTRL1 |= RSBEN;
}


/**
* @brief I2C slave clock stretch control.
*
* @param str:	0x00 = Stretch Clock
* @param 		0x40 = Release Clock
* @return none
*
* 
*/
static __INLINE void I2C_Slave_Clock_Stretch ( uint8_t str )
{
	*I2CCTRL1 = ( *I2CCTRL1 & ( ~CLK_STRETCH_OFF ) ) | (str+STRSTRE);
}


/**
* @brief I2C slave enables/disables general call.
*
* @param str:	GC_ON = General Call Enabled
* @param 		GC_OFF = General Call disabled
* @return none
*
* 
*/
static __INLINE void I2C_Slave_Enable_General_Call ( uint8_t gc )
{
	*I2CCTRL1 = ( *I2CCTRL1 & ( ~GC_ON ) ) | gc;
}


/**
* @brief I2C master issues a start bit
*
* @param none
* @return none
*
* 
*/
static __INLINE void I2C_Master_Receive_Start ( void )
{
	*I2CCTRL1 |= RCVSTRT;
}

/**
* @brief I2C master sets ack/nack
*
* @param mode: 
* 				SEND_ACK
				SEND_NACK
* @return none
*
* 
*/
static __INLINE void I2C_Master_Sets_AckNack ( uint8_t an )
{
	*I2CCTRL1 = ( *I2CCTRL1 & ( ~SET_NACK ) ) | an;
}


/**
* @brief I2C master issues an ack/nack
*
* @param none
* @return none
*
* 
*/
static __INLINE void I2C_Master_Issue_AckNack ( void )
{
	*I2CCTRL1 |= ACKEN;
}


/**
* @brief I2C master issues a stop bit
*
* @param none
* @return none
*
* 
*/
static __INLINE void I2C_Master_Issue_Stop_Bit ( void )
{
	*I2CCTRL1 |= STPSIZE;
}

/**
* @brief I2C slave sets address size
*
* @param mode: 
* 				10_BIT_ADDRESS
				7_BIT_ADDRESS
* @return none
*
* 
*/
static __INLINE void I2C_Slave_Sets_Address_Size ( uint8_t ads )
{
	*I2CCTRL1 = ( *I2CCTRL1 & ( ~STPSIZE ) ) | ads;
}


/**
* @brief I2C master issues a start bit.
*
* @param none
* @return none
*
* 
*/
static __INLINE void I2C_Master_Issue_Start_Bit ( void )
{
	*I2CCTRL1 |= STRSTRE;
}

#define I2CCTRL2    ((volatile uint8_t *) (I2C_BASE + 0x02))
#define	I2CEN		0x04
#define	I2C_ON		0x04
#define	I2C_OFF		0x00
#define	I2CFILTER	0x02
#define	I2C_FILTER_ON	0x02
#define	I2C_FILTER_OFF	0x00
#define	I2C_MASTER	0x01
#define	I2C_SLAVE	0x00

/**
* @brief I2C slave address masking.
*
* @param I2cmask[4...0]
* @return none
*
* 
*/
static __INLINE void I2C_Slave_Address_Mask ( uint8_t mask )
{
	*I2CCTRL2 = ( *I2CCTRL2 & ( ~0xF8 ) ) | (mask<<3);
}


/**
* @brief I2C Enable
*
* @param I2C_ON
* @param I2C_OFF
* @return none
*
* 
*/
static __INLINE void I2C_Enable( uint8_t st )
{
    *I2CCTRL2 = ( *I2CCTRL2 & ( ~I2CEN ) ) | (st);
    *PORTEOE |= MODEI2C; 
}


/**
* @brief I2C Filter Control
*
* @param I2C_FILTER_ON
* @param I2C_FILTER_OFF
* @return none
*
* 
*/
static __INLINE void I2C_Filter_Enable( uint8_t ft )
{
	*I2CCTRL2 = ( *I2CCTRL2 & ( ~I2CFILTER ) ) | (ft);
}



/**
* @brief I2C Mode (Master/Slave)
*
* @param mode:	I2C_MASTER
* @param 		I2C_SLAVE
* @return none
*
* 
*/
static __INLINE void I2C_Mode( uint8_t mode )
{
	*I2CCTRL2 = ( *I2CCTRL2 & ( ~I2C_MASTER ) ) | (mode);
}

#define I2CDATA     ((volatile uint8_t *) (I2C_BASE + 0x03))
#define I2CADDR0    ((volatile uint8_t *) (I2C_BASE + 0x04))
//In Master mode, this register contains the 8-bit clock frequency divider 
//counter maximum value.In Slave mode, this register contains lower 8-bit I2C address.
#define I2CADDR1    ((volatile uint8_t *) (I2C_BASE + 0x05))
//Lower two bits contain address[9....8]

/**
* @brief I2C Read Data
*
* @param none
* @return data
*
* 
*/
static __INLINE uint8_t I2C_ReadData( void )
{
  return (*I2CDATA);
}

/**
* @brief I2C Write Data
*
* @param data
* @return data
*
* 
*/
static __INLINE void I2C_WriteData( uint8_t data )
{
  *I2CDATA = data;
}

/**
* @brief I2C Write Address
*
* @param data
* @return data
*
* 
*/
static __INLINE void I2C_WriteAddress( uint16_t add )
{
  *I2CADDR0 = (uint8_t) add;
  *I2CADDR1 = (uint8_t) (add>>8);
}

/**
* @brief I2C Write Clock frequency divider
*
* @param div
* @return none
* NOTE; Fbr = Fs/(2*(1+div)) where:	Fbr is the frequency of the i2c
* 									Fs is the system clock
*									div is the divider being applied
*/
static __INLINE void I2C_WriteClockDivider( uint8_t div )
{
  *I2CADDR0 = div;
}

// **********************************************
// ADC Related Registers 
// **********************************************

#define ADC1THRESH   ((__IO uint8_t  *) (ADC_BASE + 0x00))
#define ADC1PERIOD   ((__IO uint8_t  *) (ADC_BASE + 0x01))
#define ADC2THRESH   ((__IO uint8_t  *) (ADC_BASE + 0x02))
#define ADC2PERIOD   ((__IO uint8_t  *) (ADC_BASE + 0x03))
#define ADCCHANNELS  ((__IO uint8_t  *) (ADC_BASE + 0x04))
#define ADC2_PC0     (0x00<<4)   //ADC2 (12 channels)
#define ADC2_PC1     (0x01<<4)
#define ADC2_PC2     (0x02<<4)
#define ADC2_PC3     (0x03<<4)
#define ADC2_PC4     (0x04<<4)
#define ADC2_PC5     (0x05<<4)
#define ADC2_PC6     (0x06<<4)
#define ADC2_PC7     (0x07<<4)
#define ADC2_PA3     (0x08<<4)
#define ADC2_PB5     (0x09<<4)
#define ADC2_PB6     (0x0A<<4)
#define ADC2_PB7     (0x0B<<4)
#define ADC1_PA0     (0x00<<0)   //ADC1 (15 channels)
#define ADC1_PA1     (0x01<<0)
#define ADC1_PA2     (0x02<<0)
#define ADC1_PA4     (0x04<<0)
#define ADC1_PA5     (0x05<<0)
#define ADC1_PA6     (0x06<<0)
#define ADC1_PA7     (0x07<<0)
#define ADC1_PB0     (0x08<<0)
#define ADC1_PB1     (0x09<<0)
#define ADC1_PB2     (0x0A<<0)
#define ADC1_PB3     (0x0B<<0)
#define ADC1_PB4     (0x0C<<0)

#define ADCSTART     ((__IO uint8_t  *) (ADC_BASE + 0x05))
#define ADC1DATA     ((__IO uint8_t  *) (ADC_BASE + 0x06))
#define ADC2DATA     ((__IO uint8_t  *) (ADC_BASE + 0x07))
#define	PWMFUSE		 ((__IO uint8_t  *) (ADC_BASE + 0x08))	//PWMs Fuse Control, mapped in this space!!
#define PWM1_CC_FLAG        0x80        //Short Circuit Detection Flag for PWM1
#define PWM2_CC_FLAG        0x40        //Short Circuit Detection Flag for PWM2    
#define FUSE_FLAG           0x20        //Fuse Circuit Detection Flag
#define FUSE_ENABLE         0x10        //FUSE enable and disable 
#define FUSE_DISABLE        0x00
#define PWM1_SNS_LVL1       0x08        //PWM1 sense level (1 or 0)
#define PWM1_SNS_LVL0       0x00
#define PWM2_SNS_LVL1       0x04        //PWM2 sense level (1 or 0)    
#define PWM2_SNS_LVL0       0x00
#define FUSE_OFFSET_190     0x00        //Fuse offset = 190mV 
#define FUSE_OFFSET_360     0x01        //Fuse offset = 360mV 
#define FUSE_OFFSET_670     0x02        //Fuse offset = 670mV 
#define FUSE_OFFSET_860     0x03        //Fuse offset = 860mV 


#define ADCCLKDIV    ((__IO uint8_t  *) (ADC_BASE + 0x0A)) 

#define ADC1  0x40
#define ADC2  0x80

/**
* @brief ADC Select Channels
* @param adc1ch
* @param adc2ch
* @return none
*/
static __INLINE void ADC_SelectChannel( uint8_t adc1ch, uint8_t adc2ch )
{ 
  *ADCCHANNELS = adc1ch | adc2ch;
}

/**
* @brief ADC Conversion in progress function
* @param none
* @return 0 = completed, not in progress; !0 = conversion executing
*/
static __INLINE uint8_t ADC_ConversionInProgress( void )
{ 
  return (*ADCSTART & 0x01);
}

/**
* @brief ADCs Start conversion 
* @param none
* @return none
*/
static __INLINE void ADC_Start( void )
{ 
  *ADCSTART = 0x01;
}

/**
* @brief ADCs clock divider
* @param div: 1,2,4,8,16
* @return none
*/
static __INLINE void ADC_ClkDiv( uint8_t div )
{ 
  switch (div)
  {
    case 1:   *ADCCLKDIV = 0x04;break;
    case 2:   *ADCCLKDIV = 0x00;break;
    case 4:   *ADCCLKDIV = 0x01;break;
    case 8:   *ADCCLKDIV = 0x02;break;
    case 16:  
    default:  *ADCCLKDIV = 0x03;break;
  }
}

/**
* @brief ADCs read result 
* @param number
* @return value
*/
static __INLINE uint8_t ADC_Read( uint8_t ch )
{ 
  uint8_t value;
  if ( ch == ADC1 ) value = *ADC1DATA;
  if ( ch == ADC2 ) value = *ADC2DATA;
  return (value);
}

/**
* @brief ADC threshold and period
* @param ch: channel
* @param th: threshold
* @param per: period
* @return none
*/
static __INLINE void ADC_ProgramShortCircuit ( uint8_t ch, uint8_t th, uint8_t per)
{ 
  if ( ch == ADC1 )
  {
    *ADC1THRESH = th;
    *ADC1PERIOD = per;
  }
  else if ( ch == ADC2 )
  {
    *ADC2THRESH = th;
    *ADC2PERIOD = per;   
  }
}

#define ADCREG0  ((__IO uint8_t  *) (ADC_SAP_BASE + 0x0000)) 
// ADC0 => { PHI1[3:0], PLO1[3:0]}

#define ADCREG1   ((__IO uint8_t  *) (ADC_SAP_BASE + 0x0001)) 
// ADC1 => { PHI2[3:0], PLO2[3:0]}

#define ADCREG2   ((__IO uint8_t  *) (ADC_SAP_BASE + 0x0002)) 
// ADC => { PGN1[3:0], PGN2[3:0]}

#define ADCREG3   ((__IO uint8_t  *) (ADC_SAP_BASE + 0x0003)) 
// ADC3 => { 1b'0, ADC_ENABLE, ADC_GNDOFF, ADC_SW[1:0], ADC_CAL, ADC2_FULRG, ADC1_FULRG }
#define ADCEN   0x50          //Enable ADCs and also double sampling with ADC_SW[1:0] = 10
#define ADCDIS  0x10          //Disable ADCs and also double sampling with ADC_SW[1:0] = 10
#define ADC2_REFVDD 0x02
#define ADC2_REFBG  0x00
#define ADC1_REFVDD 0x01
#define ADC1_REFBG  0x00
#define ADC_REFVDD  0x01
#define ADC_REFBG   0x00

/**
* @brief ADC Enable
*
* @param mode - ADCEN or ADCDIS
* @return NONE
* 
*/
static __INLINE void ADC_Enable( uint8_t mode)
{ 
	*ADCREG3 = (*ADCREG3 & 0xBF) | mode;
}
/**
* @brief ADC References Configuration
*
* @param ad - ADC selected: ADC1 or ADC2
* @param hr - ADCREFHG value
* @param lr - ADCREFLG value
* @param pgn - ADCPGN value
* @param ref - reference selection: ADC_REFVDD or ADC_REFBG
* @return NONE
* 
*/

static __INLINE void ADC_Reference_Config( uint8_t ad,  uint8_t hr,  uint8_t lr, uint8_t pgn, uint8_t ref)
{ 
  if ( ad == ADC1 )
  {
    *ADCREG0 = (hr<<4) + lr;
    *ADCREG2 = (*ADCREG2 & 0x0F) | (pgn<<4);
    *ADCREG3 = (*ADCREG3 & 0xFE) | ref;
  }
  else if ( ad == ADC2 )
  {
    *ADCREG1 = (hr<<4) + lr;
    *ADCREG2 = (*ADCREG2 & 0xF0) | pgn;
    *ADCREG3 = (*ADCREG3 & 0xFD) | (ref<<1);   
  }
}

#define VREGTRIM   ((__IO uint8_t  *) (ASIC_16B_BASE + 0x000B))
// VREGTRIM => { PCTRLIT , RDIV[2:0] , CZ[3:0] }

#define VREGCTRL   ((__IO uint8_t  *) (ASIC_16B_BASE + 0x000B))
// { RESERVED[7:0] }

#define PATRIM     ((__IO uint8_t  *) (ASIC_16B_BASE + 0x000C))
// PATRIM => { RCAL[1:0] , PACOREITRIM , PADRV2TRIM , PACORECCAL[3:0] }

#define PACTRL1     ((__IO uint8_t  *) (ASIC_16B_BASE + 0x000C))
// PACTRL1 => { RESERVED[7...0] }


// ****************************************************************************
// TIMERS RELATED REGISTERS
// ****************************************************************************

#define TMR0REG  ((__IO uint32_t  *) (TIMER_BASE + 0x0000))
// {T0REG[31:0] }

#define TMR0CTRL  ((__IO uint32_t  *) (TIMER_BASE + 0x0004))
// {RESERVED[31:1], T0START}

#define TMR1REG  ((__IO uint32_t  *) (TIMER_BASE + 0x0008))
// {T1REG[31:0] }

#define TMR1CTRL  ((__IO uint32_t  *) (TIMER_BASE + 0x000C))
// {RESERVED[31:1], T1START}

#define TMR2REG  ((__IO uint32_t  *) (TIMER_BASE + 0x0010))
// {T2REG[31:0] }

#define TMR2CTRL  ((__IO uint32_t  *) (TIMER_BASE + 0x0014))
// {RESERVED[31:1], T2START}
#define	TIMERON 	0x00000001
#define	TIMEROFF 	0x00000000

/**
* @brief Initializes the timers
*
* @param timernumber - the number of the timer to be addressed (0,1,2)
* @param mode - TIMERON or TIMEROFF
* @param timervalue - initial value. The timer will count from timervalue
* @param up to overflow. (0xFFFFFFFF --> 0x00000000)
* @return none
*
*  
*/
static __INLINE void TMR_Config( uint8_t timernumber, uint8_t mode, uint32_t timervalue )
{
	if ( timernumber == 0 )
	{
		*TMR0REG = timervalue;
		*TMR0CTRL = mode;
	}
	else if ( timernumber == 1 )
	{
		*TMR1REG = timervalue;
		*TMR1CTRL = mode;
	}
	else
	{
		*TMR2REG = timervalue;
		*TMR2CTRL = mode;
	}
}


// ****************************************************************************
// WDT (Watch Dog Timer)  RELATED REGISTERS
// ****************************************************************************

#define WDTCTRL  ((__IO uint32_t  *) (TIMER_BASE + 0x0018))
// {RESERVED[31...5], WDTPRES[1...0], RSTFLAG, RESETEN, WDTEN}
#define	WDT_EN		    (0x01<<0)
#define	WDT_INT 	    (0x00<<1)
#define	WDT_RST	        (0x01<<1)
#define WDT_RST_FLAG    (0x01<<2)
#define	WDT13		    (0x00<<3)
#define	WDT19		    (0x01<<3)
#define	WDT22		    (0x02<<3)
#define	WDT32		    (0x03<<3)

#define WDTCLR  ((__IO uint32_t  *) (TIMER_BASE + 0x01C))
// {WCLR[31...0]}

/**
* @brief Configs WDT. Defines operating mode and timing.
*
* @param wdt_mode specifies if the wdt will trigger interrupt or reset. Options:
* @param WDT_INT or WDT_RST
* @param wdt_divider specifies the wdt divider. Options:
* @param WDT13 = (2^13)/SystemClock	
* @param WDT19 = (2^19)/SystemClock	
* @param WDT22 = (2^22)/SystemClock	
* @param WDT32 = (2^32)/SystemClock	
* @return none
*
* Selects the reset source and divider. Does not
* alter the WDT flag. Enables the peripheral.
*/
static __INLINE void WDT_Config( uint8_t wdt_mode, uint8_t wdt_divider)
{
	*WDTCTRL = wdt_mode + wdt_divider + 0x01;
}

// @brief Clears WDT.
//
// @param none
// @return none
//
// Clears (pets) the WDT. 
//
/*
static __INLINE void WDT_Clear( void )
{
  WDT->KEY = WDT_KEY_CLEAR;
  WDT->KEY = WDT_KEY_KEY0;
  WDT->KEY = WDT_KEY_KEY1;
}
*/

static __INLINE void WDT_Clear( void )
{
	*WDTCLR = 0x3C570001;
	*WDTCLR = 0x007F4AD6;
}


// @brief Reads WDT flag returning it.
//
// @param none
// @return Flag (RSTFLAG)
//
static __INLINE uint32_t WDT_GetFLag( void )
{
  return (WDT->CFG & WDT_CFG_FLAG);
}

/**
* @brief Reads WDT flag returning it.
*
* @param none
* @return Flag (RSTFLAG)
*
* 
*/
static __INLINE uint32_t WDT_ReadFLag( void )
{
	return (*WDTCTRL & 0x00000004);
}


/**
* @brief Clears WDT flag.
*
* @param none
* @return none
*
* Clears the WDT. 
*/
static __INLINE void WDT_ClearFLag( void )
{
	*WDTCTRL &= ~0x00000004;
}

// ****************************************************************************
// FLASH MEMORY RELATED REGISTERS
// ****************************************************************************

#define FLADDR  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x000))
// {FLADD[31...0]}

#define FLWRDT  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x004))
// {FLDT[31...0]}

#define UNLBWR  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x008))
// {UBW[31...0]}

#define BWRSTRT  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x00C))
// {BRST[31...0]}

#define UNLSER  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x010))
// {USE[31...0]}

#define SERSTRT  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x014))
// {SES[31...0]}

#define UNLBLKER  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x018))
// {UBKE[31...0]}

#define BKERSTRT  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x01C))
// {BES[31...0]}

#define FLSCTRL  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x020))
// {RESERVED[31...6], WECD2[5...3], RWC[2...0]}

#define FLASH_BUSY_REG            ((volatile uint8_t*)  0x50020041)


// #define FLSCP  ((__IO uint32_t  *) (FLASH_CTRL_BASE + 0x024)) // Had been defined at clough_core.h
// {FCP[31...0]}

#define FLASH_BUSY_REG           ((volatile uint8_t*)  0x50020041)

#define	PROTECT		0xF2E11047
#define	UNPROTECT	0x00000000

#define FLASH_BYTE_UNLOCK_PAT1 ((uint32_t) 0x55555555)
#define FLASH_BYTE_UNLOCK_PAT2 ((uint32_t) 0xAAAAAAAA)
#define FLASH_BLOCK_UNLOCK_PAT ((uint32_t) 0x77777777)
#define FLASH_BLOCK_ERASE_PAT  ((uint32_t) 0x88888888)
#define FLASH_STATUS_REG_BUSY  ((uint32_t) 0x00000400)

/**
* @brief Flash memory erase block function.
*
* @param address inside the block to be erased.
* @return none
*
*  
*/
static __INLINE void FLS_EraseBlock( __IO uint32_t address )
{
	*FLADDR   = address;		//Point to the block starting at address
	*UNLBLKER = 0x77777777;		//Unlock block erase 
	*BKERSTRT = 0x88888888;		//Start erase process 
}

/**
* @brief Flash memory erase block check function.
*
* @param none
* @return Block Erase Flag, 0x100 if still erasing, '0' otherwise
*
*  
*/
static __INLINE  uint32_t FLS_CheckErasingBlock( void )
{
	return ( *FLSCTRL & 0x00000400 );	//Return the Block Erase Flag
}


/**
* @brief Flash memory erase sector function.
*
* @param address inside the sector to be erased.
* @return none
*
*  
*/
static __INLINE void FLS_EraseSector( __IO uint32_t address )
{
	*FLADDR   = address;		//Point to the sector starting at address
	*UNLSER = 0x66666666;		//Unlock sector erase 
	*SERSTRT = 0x99999999;		//Start erase process 
}

/**
* @brief Flash memory erase sector check function.
*
* @param none
* @return Sector Erase Flag, 0x80 if still erasing, '0' otherwise
*
*  
*/
static __INLINE  uint32_t FLS_CheckErasingSector( void )
{
	return ( *FLSCTRL & 0x00000200 );	//Return the Sector Erase Flag
}


/**
* @brief Flash memory write byte function.
*
* @param address and data (byte) to be writen into flash.
* @return none
*
*  
*/
static __INLINE  void FLS_WriteByte( __IO uint32_t address, __IO uint8_t databyte )
{
	*FLADDR  = address;			//Point to the byte address 
	*UNLBWR  = 0x55555555;			//Unlock byte write 
	*BWRSTRT = 0xAAAAAAAA;			//Start write process
	*FLWRDT  = (uint32_t)databyte;	        //Load 0xAB to be written
}


/**
* @brief Flash memory write byte check function.
*
* @param none.
* @return Byte write Flag, 0x40 if still erasing, '0' otherwise
*
*  
*/
static __INLINE  uint32_t FLS_CheckWritingByte( void )
{
	return ( *FLSCTRL & 0x00000100 );
}


/**
* @brief Flash memory write byte check function.
*
* @param mode. Options: PROTECT or UNPROTECT
* @return none
*
*  
*/
static __INLINE  void FLS_CodeProtection( uint32_t mode )
{
    *FLSCP = mode;		//Code protection 
}

// ******************************************
// UART Related Registers
// ******************************************

#define UARTDATA    ((__IO uint8_t *) (0x50000010))	    //UART Data (TX and RX)
#define UARTICR     ((__IO uint8_t *) (0x50000011))	    //UART INT CTRL REG
#define UARTICTCL   ((__IO uint8_t *) (0x50000011))	    //UART INT CTRL REG
#define	UART_NOINT		(0x01<<4)	//No interrupt asserted
#define	UART_TXDONE		(0x02<<4)	//Transmission done
#define	UART_RXRDY		(0x04<<4)	//Data received
#define	UART_RX_ERROR	(0x06<<4)	//Error detected (Reception)
#define	UART_TIMEOUT	(0x0C<<4)	//Reception timeout (~40 bits-time)
#define	UART_TOUTEN		(0x08)		//Timeout int. enable bit
#define	UART_TOUTDIS	(0x00)      //
#define	UART_ERREN		(0x04)		//Error int. enable (RX) bit
#define	UART_ERRDIS		(0x00)		//
#define	UART_TXEN		(0x02)		//TX int. enabled
#define	UART_TXDIS		(0x00)		//
#define	UART_RXEN		(0x01)		//RX int. enabled
#define	UART_RXDIS		(0x00)		//

/**
* @brief UART Interrupt Control 
*
* @param tout: Timeout int. enable/disable - UART_TOUTEN or UART_TOUTDIS
* @param err: RX error int. enable/disable - UART_ERREN or UART_ERRDIS
* @param txe: TX int. enable/disable - UART_TXEN or UART_TXDIS
* @param rxe: RX int. enable/disable - UART_RXEN or UART_RXDIS
* @return none
*
* 
*/
static __INLINE void UART_Interrupt_Control( uint8_t tout, uint8_t err,uint8_t txe,uint8_t rxe)
{
	*UARTICR = tout | err | txe | rxe;
}

/**
* @brief UART Interrupt status
*
* @param none
* @return UART_NOINT, UART_ERROR, UART_RXRDY, UART_TIMEOUT, UART_TXDONE
*
* 
*/
static __INLINE uint8_t UART_Interrupt_Status( void )
{
	return (*UARTICR & 0xF0);
}

// Example Code
/*

	UART_Interrupt_Control(UART_TOUTEN, UART_ERREN, UART_TXEN, UART_RXEN);		// Enable timeout, error (RX), transmission and reception ints.
	
	switch(UART_Interrupt_Status())
	{
		case	UART_ERROR:
			//Process reception error here
			break;
		case 	UART_RXRDY:
			//Process data received
			mydata = *UARTDATA;	//Read data received by uart into mydata
			break;
		case	UART_TIMEOUT:
			//Process reception timeout
			break;
		case	UART_TXDONE:
			//Process transmission complete
			break;
		case 	UART_NOINT:	//No int.
		default:
				//If no interrupt asserted or something else happened nothing to do
				break;
	}

*/

#define UARTCTRL  ((__IO uint8_t *) (UART_BASE + 0x02))	//UART CTRL REG
#define	UART_LBEN		0x80	//Loop back enable
#define	UART_LBDIS		0x00	//
#define	UART_BREAKEN	0x40	//Break enable
#define	UART_BREAKDIS	0x00
#define	UART_STPEN		0x20	//Sticky parity 
#define	UART_STPDIS		0x00
#define	UART_EVENEN		0x10	//Even parity selected
#define	UART_ODDEN		0x00	//Odd parity selected
#define	UART_PAREN		0x08	//Parity Enable
#define	UART_PARDIS		0x00	
#define	UART_10STOP		0x00	//One stop-bit
#define	UART_15STOP		0x04	//One and a half stop-bits (5-bit communications only)
#define	UART_20STOP		0x04	//Two stop-bits
#define	UART_5BITS		0x00	//5-bit UART
#define	UART_6BITS		0x01	//6-bit UART
#define	UART_7BITS		0x02	//7-bit UART
#define	UART_8BITS		0x03	//8-bit UART

/**
* @brief UART Control 
*
* @param lp: Loop back control - UART_LBEN or UART_LBDIS
* @param brk: Break (Low level) - UART_BREAKEN or UART_BREAKDIS
* @param stp: sticky parity (fixed in '1') - UART_STPEN or UART_STPDIS
* @param ev: even or odd parity select - UART_EVENEN or UART_ODDEN
* @param pe: parity enable - UART_PAREN or UART_PARDIS
* @param st: number of stop bits - UART_10STOP, UART_15STOP or UART_20STOP (1, 1.5 and 2)
* @param nb: number of bits - UART_5BITS, UART_6BITS, UART_7BITS, UART_8BITS (5,6,7,8)
* @return none
* Obs: 1.5 stop bits for 5-bit comm. only!
* 
*/
static __INLINE void UART_Ctrl( uint8_t lp, uint8_t brk, uint8_t stp, uint8_t ev, uint8_t pe, uint8_t st, uint8_t nb )
{
	*UARTCTRL = lp | brk | stp | ev | pe | st | nb;
}

/**
* @brief UART Control Read
*
* @param none
* @return UARTCTRL register
* Obs: 1.5 stop bits for 5-bit comm. only!
* 
*/
static __INLINE uint8_t UART_Ctrl_Read( void )
{
	return(*UARTCTRL);
}

#define UARTCTRL1  ((__IO uint8_t *) (UART_BASE + 0x03))	//UART CTRL REG 1
#define	UARTEN			0x08	//UART enabled
#define	UARTDIS			0x00
#define	UART_INT1		0x00	//UART interrupts after 1 byte received
#define	UART_INT3		0x04	//UART interrupts after 3 bytes received 
#define	UART_TXFFRST	0x02	//TX FIFO reset
#define	UART_RXFFRST	0x01	//RX FIFO reset

/**
* @brief UART Control 
*
* @param en: uart enable - UARTEN or UARTDIS
* @param nt: number of bytes received to generate int. - UART_INT1 or UART_INT3
* @param tfr: TX fifo reset- UART_TXFFRST
* @param rfr: RX fifo reset- UART_RXFFRST
* @return none
* 
*/
static __INLINE void UART_Ctrl1( uint8_t en, uint8_t nt, uint8_t tfr, uint8_t rfr )
{
	*UARTCTRL1 = en | nt | tfr | rfr;
    if (en == UARTEN)   *PORTEOE |= MODEUART;   //If enabled set mode in port
    else    *PORTEOE &= ~MODEUART;
}

//Example Code
/*
	UART_Ctrl1( UARTEN, UART_INT3, UART_TXFFRST, 0 );		//Enable UART, interrupt after receiving 3 bytes, clear TX fifo but not RX fifo
*/
	
#define UARTSTATUS ((__IO uint8_t *) (UART_BASE + 0x04))	//UART status register
#define UART_ERROR          0x80
#define UART_TX_EMPTY       0x40
#define UART_TX_FIFO_EMPTY  0x20
#define UART_BREAK_INT      0x10
#define UART_FRAMING_ERROR  0x08
#define UART_PARITY_ERROR   0x04
#define UART_OVERRUN_ERROR  0x02
#define UART_DATA_READY     0x01

/**
* @brief UART Control 
*
* @param none
* @return '0' or (UART_FRAMING_ERROR | UART_PARITY_ERROR | UART_OVERRUN_ERROR)
* 
*/
static __INLINE uint8_t UART_CheckError ( void )
{
  return ( (*UARTSTATUS & UART_ERROR)? (*UARTSTATUS & 0x0E):  0x00 );
}

/**
* @brief UART check for TX empty 
*
* @param none
* @return '0' or UART_TX_EMPTY
* 
*/
static __INLINE uint8_t UART_TxEmpty ( void )
{
  return ( *UARTSTATUS & UART_TX_EMPTY );
}


/**
* @brief UART check for fifo empty 
*
* @param none
* @return '0' or UART_TX_FIFO_EMPTY
* 
*/
static __INLINE uint8_t UART_TxFifoEmpty ( void )
{
  return ( *UARTSTATUS & UART_TX_FIFO_EMPTY );
}


#define UARTDIV ((__IO uint16_t *) (UART_BASE + 0x06))	//UART baud rate divider

/**
* @brief UART baud rate divider 
*
* @param div
* @return none
* Note:  Baud = Fsys/(16*(div+1)). Examples:(@3.58MHz)
*	Baud	DIV	Real_Baud	Error (%)
*	300		745	299.9		0.04%
*	600		372	599.8		0.04%
*	1200	185	1202.8		0.23%
*	2400	92	2405.6		0.23%
*	4800	46	4760.0		0.83%
*	9600	22	9727.0		1.32%
*	19200	11	18643.5		2.90%
*	28800	7	27965.2		2.90%
*	38400	5	37286.9		2.90%
*	57600	3	55930.4		2.90%
*	115200	1	111860.8	2.90%
*/
static __INLINE void UART_BaudRateDivider ( uint16_t div )
{
  *UARTDIV = div;
}

// *********************
// SPI Related Registers
// *********************


#define SPCR   ((__IO uint8_t *) (0x5000001C))	//SPI control
#define SPI_INT_EN      0x80        //SPI interrupt enable
#define SPI_INT_DIS     0x00
#define SPI_MASTER      0x10        //SPI always as master in KamCho
#define SPI_CPOL_POS    0x08        //SPI Clock Polarity Positive 
#define SPI_CPOL_NEG    0x00
#define SPI_PHASE1      0x04        //SPI Clock Phase
#define SPI_PHASE0      0x00

//SPR[1...0] clock rate standard - See SPER register

#define SPSR   ((__IO uint8_t *) (SPI_BASE + 0x01))	//SPI status
#define SPI_INT_FLAG          0x80      //Interrupt Flag
#define SPI_WCOL              0x40      //Collision Flag
#define SPI_TX_FIFO_FULL      0x08      //TX FIFO full Flag
#define SPI_TX_FIFO_EMPTY     0x04      //TX FIFO empty Flag
#define SPI_RX_FIFO_FULL      0x02      //RX FIFO full Flag
#define SPI_RX_FIFO_EMPTY     0x01      //RX FIFO empty Flag

#define SPIDATA   ((__IO uint8_t *) (SPI_BASE + 0x02))	//SPI data

#define SPER   ((__IO uint8_t *) (SPI_BASE + 0x03))	//SPI extension register
//bits 7:6 = ICNT[1...0] = number of bits received in FIFO before SINTF is set
//	00 = SINTF is set after every completed transfer
//	01 = SINTF is set after two completed transfers
//	10 = SINTF is set after three completed transfers
//	11 = SINTF is set after four completed transfers
#define SPI_EN  0x04        //SPI Enable Flag
#define SPI_DIS 0x04        //
//SPR[1...0]:  Clock phase extended
//SCKEXT[1:0]: SPI extended clock divider
//	SCKSTD	SCKEXT 	Result Clock Divider
//	00		00 		= System Clock/2
//	01		00 		= System Clock/4
//	10		00		= System Clock/16
//	11		00 		= System Clock/32
//  00		01 		= System Clock/8
//	01		01 		= System Clock/64
//	10		01		= System Clock/128
//	11		01 		= System Clock/256
//	00		10 		= System Clock/512
//	01		10 		= System Clock/1024
//	10		10		= System Clock/2048
//	11		10 		= System Clock/4096
//  xx		11 		= Reserved

/**
* @brief SPI Config
*
* @param inten: SPI_INT_EN or SPI_INT_DIS
* @param en: SPI_EN or SPI_DIS
* @param pol: SPI_CPOL_POS or SPI_CPOL_NEG
* @param ph: SPI_PHASE1 or SPI_PHASE0
* @param div: Divider --> 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096
*
* 
*/
static __INLINE void SPI_Config( uint8_t inten,  uint8_t en,  uint8_t pol,  uint8_t ph,  uint8_t div )
{
  *SPCR = inten | pol | ph;
  if (en == SPI_EN) *PORTEOE |= MODESPI;    //If enabling set enabling at port
  else  *PORTEOE &= ~MODESPI;
  *SPER = (*SPER & 0xF8) | en;              //Enable interface if necessary
  switch (div)                              //Set Divider
  {
    case 2:                               break;
    case 4:       *SPCR |= 0x01;          break;   
    case 16:      *SPCR |= 0x02;          break;   
    case 32:      *SPCR |= 0x03;          break;   
    case 8:                                       break;
    case 64:      *SPCR |= 0x01; *SPER |= 0x01;   break;   
    case 128:     *SPCR |= 0x02; *SPER |= 0x01;   break;   
    case 256:     *SPCR |= 0x03; *SPER |= 0x01;   break;       
    case 512:                                     break;
    case 1024:    *SPCR |= 0x01; *SPER |= 0x02;   break;   
    case 2048:    *SPCR |= 0x02; *SPER |= 0x02;   break;   
    case 4096:
    default:      *SPCR |= 0x03; *SPER |= 0x02;   break;   
  }
}

/**
* @brief SPI read status
*
* @param None
* @return (SPI_INT_FLAG | SPI_WCOL | SPI_TX_FIFO_FULL | SPI_TX_FIFO_EMPTY | SPI_RX_FIFO_FULL | SPI_RX_FIFO_EMPTY)
*/
static __INLINE uint8_t SPI_ReadStatus( void )
{
  return (*SPSR );
}

static __INLINE void SPI_INT_Clear( void )
{
  *SPSR |= SPI_INT_FLAG;       // Bit 7 should be written 1 to clear SINTF
}

/**
* @brief SPI Rx Fifo Size
*
* @param None
* @return RX fifo size (1,2,3,4)
*/
static __INLINE uint8_t SPI_ReadRxFifoSize( void )
{
  return ((*SPER>>6)+1);
}

// ******************************************
// LIN Related Registers
// ******************************************

#define LIN_DATA0  ((__IO uint8_t *) (LIN_BASE + 0x00))	//LIN Data defined in bytes
#define LIN_DATA1  ((__IO uint8_t *) (LIN_BASE + 0x01))	//
#define LIN_DATA2  ((__IO uint8_t *) (LIN_BASE + 0x02))	//
#define LIN_DATA3  ((__IO uint8_t *) (LIN_BASE + 0x03))	//
#define LIN_DATA4  ((__IO uint8_t *) (LIN_BASE + 0x04))	//
#define LIN_DATA5  ((__IO uint8_t *) (LIN_BASE + 0x05))	//
#define LIN_DATA6  ((__IO uint8_t *) (LIN_BASE + 0x06))	//
#define LIN_DATA7  ((__IO uint8_t *) (LIN_BASE + 0x07))	//



#define LINDATA  ((__IO LinDataType *) (LIN_BASE + 0x00))	//LIN Data structure address

/**
* @brief LIN Interface Data Read
* @param : none
* @return : Pointer to data byte [0]
* Note: 
*/
static __INLINE uint8_t *LIN_ReadData( void )
{
	return ((uint8_t *)&LINDATA->BYTE[0]);
}
	
#define LINCTRL  ((__IO uint8_t *) (LIN_BASE + 0x08))	//LIN Control
#define	LIN_SLAVE_STOP				0x80	//
#define	LIN_SLEEP					0x40
#define	LIN_SLEEP_ENABLE			0x40
#define	LIN_SLEEP_DISABLE			0x00
#define	LIN_TRANSMIT_ENABLE			0x20	//LIN transmit Enable
#define	LIN_RECEIVE_ENABLE			0x00		
#define	LIN_SLAVE_DATA_ACK			0x10
#define	LIN_SLAVE_DATA_NACK			0x00
#define	LIN_RESET_INT				0x08
#define	LIN_RESET_ERROR				0x04
#define	LIN_WAKEUP_REQUEST			0x02
#define	LIN_MASTER_START_REQUEST	0x01

/**
* @brief LIN slave stop command
* @param none
* @return : none
* Note: (Slave mode only) The host has to write a ??to this bit if it handles
*		a data request interrupt and cannot make use of the frame content with 
* 		the received identifier (e.g. extended identifiers). For that case the 
*		LIN slave stops the processing of the LIN communication until the next 
*		SYNC BREAK is detected.
*/
static __INLINE void LIN_SlaveStop( void )
{
	*LINCTRL |= LIN_SLAVE_STOP;
}

/**
* @brief LIN Check Sleep Mode
* @param none
* @return : Sleep Mode 
* Note: The bit is used by the LIN core to determine whether the LIN bus
*		is in Sleep Mode or not.
*/
static __INLINE uint8_t LIN_CheckSleep( void )
{
	return (*LINCTRL & LIN_SLEEP);
}

/**
* @brief LIN Set Sleep Mode
* @param mode: LIN_SLEEP_ENABLE or LIN_SLEEP_DISABLE
* @return : none
* Note: The host controller has to set the bit after 
*		sending or receiving a Sleep Mode frame or if a bus idle timeout 
*		interrupt is requested. The bit will be reset by the LIN core when 
*		a wakeup signal is detected.
*/
static __INLINE void LIN_SetSleep( uint8_t mode )
{
	*LINCTRL = (*LINCTRL & (~LIN_SLEEP)) | mode;
}

/**
* @brief LIN Mode TX or RX
* @param mode: LIN_TRANSMIT_ENABLE or LIN_RECEIVE_ENABLE
* @return : none
* Note: The bit determines whether the current frame is a transmit frame
*		or a receive frame for the LIN node. It has to be set by the host controller. 
*/
static __INLINE void LIN_SetMode( uint8_t mode )
{
	*LINCTRL = (*LINCTRL & (~LIN_TRANSMIT_ENABLE)) | mode;
}


/**
* @brief LIN Slave Data Ack
* @param mode: LIN_SLAVE_DATA_NACK	or LIN_SLAVE_DATA_ACK	
* @return : none
* Note: (Slave mode only) The bit has to be set by the host controller after 
*       handling a data request interrupt (compare bit LIN_DATA_REQ in LIN_STATUS 
*		register). The bit will be reset by the LIN core.
*/
static __INLINE void LIN_SlaveDataAck( uint8_t mode )
{
	*LINCTRL = (*LINCTRL & (~LIN_SLAVE_DATA_ACK)) | mode;
}

/**
* @brief LIN Reset Interrupt
* @param none	
* @return : none
* Note: The host controller has to write a ??to this bit to reset the INT bit 
*       in the status register.
*/
static __INLINE void LIN_ResetInterrupt ( void )
{
	*LINCTRL |= LIN_RESET_INT;
}

/**
* @brief LIN Reset Error
* @param none	
* @return : none
* Note: The host controller has to write a ??to this bit to reset the error
*		bits in status register and error register.
*/
static __INLINE void LIN_ResetError ( void )
{
	*LINCTRL |= LIN_RESET_ERROR;
}

/**
* @brief LIN wake up from sleep function
* @param none	
* @return : none
* Note: The bit has to be set by the host controller to terminate the Sleep
*       Mode of the LIN bus by sending a Wakeup signal. The bit will be reset 
*       by the LIN core.
*/
static __INLINE void LIN_WakeUp ( void )
{
	*LINCTRL |= LIN_WAKEUP_REQUEST;
}


/**
* @brief LIN Master Start Transmission
* @param none	
* @return : none
* Note: (Master mode only) The bit has to be set by the host controller to 
*       start the LIN transmission after loading Identifier, data length 
*		and data buffer. The LIN core will reset the bit after the transmission
*		is finished or an error is occurred.
*/
static __INLINE void LIN_MasterStartTransmission ( void )
{
	*LINCTRL |= LIN_MASTER_START_REQUEST;
}

#define LINSTATUS  ((__IO uint8_t *) (LIN_BASE + 0x09))	//LIN Control
#define LIN_ACTIVE            0x80
#define LIN_BUS_IDLE_TIMEOUT  0x40
#define LIN_ABORTED           0x20
#define LIN_DATA_REQ          0x10
#define LIN_INT_REQ           0x08
#define LIN_ERROR             0x04
#define LIN_WAKEUP            0x02
#define LIN_COMPLETE          0x01


/**
* @brief LIN Read Status
* @param none	
* @return : Status of the LIN Interface defined by the following bits:
*LIN_ACTIVE    
*   Indicates whether the LIN bus is active or not. 
*   1 = transmission on the LIN bus is active. 
*   0 = no LIN bus activity. 
*   Warning: For the LIN slave, this bit is set after the 
*   detection of a correct SYNC BREAK / SYNC FIELD sequence and 
*   it is reset at the end of the transmission or if the 
*   processing of the current frame is stopped by the host controller.
*LIN_BUS_IDLE_TIMEOUT
*   (Slave mode only) This bit is set by the LIN core 
*   if bit LIN_SLEEP in LIN_CONTROL register is not set 
*   and no bus activity is detected for 4 s. In addition, 
*   an interrupt request to the host controller is 
*   generated in that case. After that, the host controller 
*   may assume that the LIN bus is in sleep mode and it 
*   has to set bit LIN_SLEEP in LIN_CONTROL register of 
*   the LIN core.  
*LIN_ABORTED  
*   (Slave mode only) This bit is set by the LIN core slave if a transmission is 
*   aborted after the beginning of the data field due to a timeout or bit error 
*   (caused e. g. by a new sync break after missing data bytes). The bit is also set 
*   if the processing of the current frame has been stopped by writing a ??to 
*   LIN_STOP in LIN_CONTROL register. The bit is cleared by the LIN core after 
*   receiving a correct SYNC BREAK / SYNC FIELD sequence.
*LIN_DATA_REQ   
*   The LIN core slave sets the bit after receiving the Identifier and sends an 
*   interrupt to the host controller. The host controller has to decode the 
*   Identifier to decide whether the current frame is a transmit or a receive 
*   operation. It has to adjust the TRANSMIT bit in the control register and to 
*   load the data length. For transmit operations it has to load the data buffer 
*   too. After that the host controller has to set the bit DATA_ACK in LIN_CONTROL register.
*LIN_INT_REQ 
*   The LIN core sets the bit when it sends an interrupt to the host controller. 
*   The bit has to be reset by the host controller by setting LIN_RESET_INT in 
*   LIN_CONTROL register.
*LIN_ERROR
*   The LIN core sets the bit if an error has been detected (compare error register). 
*   The bit has to be reset by the host controller by setting the bit LIN_RESET_ERROR \
*   in LIN_CONTROL register.
*LIN_WAKEUP         
*   The bit is set when the LIN core is transmitting a Wakeup signal or when the 
*   LIN core has received a Wakeup signal.
*LIN_COMPLETE     
*   The LIN core will set the bit after a transmission has been successfully 
*   finished and it will reset it at the start of a transmission.
*/
static __INLINE uint8_t LIN_ReadStatus ( void )
{
  return ( *LINSTATUS );
}


#define LINERROR  ((__IO uint8_t *) (LIN_BASE + 0x0A))	//LIN Error codes
#define   LIN_PARITY_ERROR    0x08
#define   LIN_TIMEOUT_ERROR   0x04
#define   LIN_CHECKSUM_ERROR  0x02
#define   LIN_BIT_ERROR       0x01

/**
* @brief LIN Error Codes Read Function
* @param none	
* @return : errors 
* NOTES: BIT ERROR -  The Bit value monitored on the bus is different from the 
*                     sent bit value when set.
*        TIMEOUT_ERROR - There are several reason that can cause a timeout error:
*                     The master detects a timeout error if it is expecting data 
*                     from the bus but no slave does respond. If the slave 
*                     responds to late and the frame is not finished within the 
*                     maximum frame length TFRAME_MAX a timeout error will be detected too. 
*                     The slave detects a timeout error if it is requesting a data 
*                     acknowledge to the host controller (for selecting receive 
*                     or transmit, data length and loading data) and the host 
*                     controller does not set the bit DATA_ACK or bit STOP in control 
*                     register until the end of the reception of the first byte 
*                     after the identifier. The slave detects a timeout error if 
*                     it has transmitted a wakeup signal and it detects no sync 
*                     field (from the master) within 150 ms. 
*                     Warning: The slave does not perform an exact check of the 
*                     frame length TFRAME_MAX but a timeout is detected after 200 bit 
*                     times if the slave is in receive mode and there are missing 
*                     data fields or a missing ID field from the master.
*/
static __INLINE uint8_t LIN_ReadErrors ( void )
{
  return(*LINERROR&0x0F);
}


#define LINLENGTH  ((__IO uint8_t *) (LIN_BASE + 0x0B))	//LIN Control
#define ENHANCED_CHECKSUM 0x80
#define CLASSIC_CHECKSUM  0x00
#define LIN_TRANSCEIVER_ACTIVE_LOW  0x40
#define LIN_TRANSCEIVER_ACTIVE_HIGH 0x00


/**
* @brief LIN Interface Data Write
* @param pd: Pointer to data to be writen
* @return : none
* Note: The function writes the number of bytes defined in LIN_DATA_LENGTH
*/
static __INLINE void LIN_WriteData( uint8_t *pd )
{
	uint8_t i = 0;
	
	for ( i = 0; i < (*LINLENGTH & 0x0F); i++ )	LINDATA->BYTE[i] = pd[i];
}
	

/**
* @brief LIN Selects Checksum
* @param chk: ENHANCED_CHECKSUM or CLASSIC_CHECKSUM
* @return : none
* Note: Enhanced checksum (Spec 2.0, inverted eight bit sum with carry over all 
*       data bytes and protected identifier).
*       Classic checksum (Spec 1.3, inverted eight bit sum with carry over all 
*       data bytes)
*/
static __INLINE void LIN_SelectChecksum ( uint8_t chk )
{
  *LINLENGTH = (*LINLENGTH & (~ENHANCED_CHECKSUM)) | chk;
}


/**
* @brief LIN Selects Transceiver active polarity
* @param chk: LIN_TRANSCEIVER_ACTIVE_LOW or LIN_TRANSCEIVER_ACTIVE_HIGH
* @return : none
* Note: 
*/
static __INLINE void LIN_Select_Transceiver_Active_Polarity ( uint8_t pol )
{
  *LINLENGTH = (*LINLENGTH & (~LIN_TRANSCEIVER_ACTIVE_LOW)) | pol;
}

/**
* @brief LIN selects data length
* @param len: 
* @return : none
* Note: The host controller has to define the length of the data field of the 
*       current LIN frame by adjusting the DATA_LENGTH register. If the 
*       variable is loaded with the value ?111b?the length of the data
*       field is decoded from Bit 5 and 4 of the identifier register id?
*       according to table below:
*       id[5...4] Number of Bytes
*       00        2
*       01        2
*       10        4
*       11        8
*/
static __INLINE void LIN_SelectDataLength ( uint8_t len )
{
  *LINLENGTH = (*LINLENGTH & (0xF0)) | len;
}

#define LINBITDIV ((__IO uint8_t *) (LIN_BASE + 0x0C))	//LIN Bit timing mult. and divider
#define LINBITMUL ((__IO uint8_t *) (LIN_BASE + 0x0D))	//

#define LINID ((__IO uint8_t *) (LIN_BASE + 0x0E))	//


/**
* @brief LIN checksum, transceiver and data size selection
* @param chk - checksum: CLASSIC_CHECKSUM or ENHANCED_CHECKSUM
* @param tran - transceiver polarity: LIN_TRANSCEIVER_ACTIVE_LOW or LIN_TRANSCEIVER_ACTIVE_HIGH
* @param len - data length
* NOTE: :	The application has to define the length of the data field of the 
*			current LIN frame by adjusting the LINDLEN[3...0] bits. If the 
*			bits are loaded with the value 	?111b?the length of the data 
*			field is decoded from Bit 5 and 4 of the identifier register id?
*			according to the table below (e.g. compatibility to LIN specification 1.1).
*			Otherwise the amount of data bytes can be written directly to the 
*			register (supported values are 0..8). 
*	Table 8 LIN data length (when the length bits have the value ?111b?
	ID Bit 5	ID Bit 4	Number of Bytes
		0			0			2
		0			1			2
		1			0			4
		1			1			8
* @return : none 
*/
static __INLINE void LIN_Set_Chk_Tran_length ( uint8_t chk, uint8_t tran, uint8_t len )
{
  *LINLENGTH = chk | tran | len;
}


/**
* @brief LIN ID selection
* @param ID	
* @return : none 
*/
static __INLINE void LIN_SetID ( uint8_t id )
{
  *LINID = id;
}

/**
* @brief LIN Multiplier and Divider Settings
* @param mul
* @param Div
* @return : none 
*/
static __INLINE void LIN_SetMultDiv ( uint8_t mul, uint16_t div)
{
  *LINBITDIV = (uint8_t) div;
  *LINBITMUL = (mul<<1) + (uint8_t)(div>>8);
}

#define LINTIMING ((__IO uint8_t *) (LIN_BASE + 0x0F))	//
#define LIN_MASTER  0x80			//Master/Slave mode selection bit
#define LIN_SLAVE   0x00
#define	LIN_DIV_FRAC0	(0x00<<4)	//Fractional divider values
#define	LIN_DIV_FRAC1	(0x01<<4)
#define	LIN_DIV_FRAC2	(0x02<<4)
#define	LIN_DIV_FRAC3	(0x03<<4)
#define	LIN_DIV_FRAC4	(0x04<<4)
#define	LIN_DIV_FRAC5	(0x05<<4)
#define	LIN_DIV_FRAC6	(0x06<<4)
#define	LIN_DIV_FRAC7	(0x07<<4)
#define	LIN_INACT4SEC	(0x00<<2)
#define	LIN_INACT6SEC	(0x01<<2)
#define	LIN_INACT8SEC	(0x02<<2)
#define	LIN_INACT10SEC	(0x03<<2)
#define	LIN_WPRPT180MS	(0x00)
#define	LIN_WPRPT200MS	(0x01)
#define	LIN_WPRPT220MS	(0x02)
#define	LIN_WPRPT240MS	(0x03)
/**
* @brief LIN Select Master or Slave Modes
* @param mode: LIN_MASTER or LIN_SLAVE
* @return : none 
*/
static __INLINE void LIN_MasterSlave ( uint8_t mode )
{
  *LINTIMING = (*LINTIMING & 0x7F) | mode;
}

#define LIN_GPIO_EN     MODELIN
#define LIN_GPIO_DIS    ~MODELIN    

/**
* @brief LIN GPIO control
* @param mode: LIN_GPIO_EN or LIN_GPIO_DIS
* @return : none 
*/
static __INLINE void LIN_Gpio_Control ( uint8_t mode )
{
  if (mode == LIN_GPIO_EN)  *PORTEOE |= LIN_GPIO_EN;
  else  *PORTEOE &= LIN_GPIO_DIS;
}

/**
* @brief LIN Fractional divider, inactive time and repeat time select function
* @param fdiv: LIN_DIV_FRACn (n = 0,1,2,...7)
* @param inac: LIN_INACT4SEC, LIN_INACT6SEC, LIN_INACT8SEC, LIN_INACT10SEC
* @param repeat: LIN_WPRPT180MS, LIN_WPRPT200MS, LIN_WPRPT220MS, LIN_WPRPT240MS
* @return : none 
*/
static __INLINE void LIN_TimingControl ( uint8_t fdiv, uint8_t inac, uint8_t repeat )
{
  *LINTIMING = (*LINTIMING & 0x80) | (fdiv<<4) | (inac<<2) | repeat;
}


#endif
