/**
* @brief ADCs Test Software for uSesame
*
* 1/23/2012
* By: David Chen
* 
*/

#include <stdint.h>
#include "uSesame.h"

        
//************** Prototype *******************

uint8_t XTCLK_Init ( void );
void RC10MCLK_Init ( void );

//************** Functions *******************

/*
* @param 	RC10KHZ - Clock source = RC AUX 10KHz
* @param 	RC10MHz - Clock source = RC 10MHz
* @param 	XTCLOCK - Clock source = XT       */

uint8_t XTCLK_Init ( void )
{
  int i,j;
  
  // Initialize the Crystal Clock to run the system, divided by one (default)
  j=0;
  if (!(*PMUCLK & (XTCLOCK|RC10MHZ))) CLK_SelectClockSource( RC10MHZ ); // Switch 10K to 10MHz
  while ((*PMUCLK & (XTCLOCK|RC10MHZ)) != XTCLOCK )  		//Wait for XT ON amd stable
  {
    if (!(*PMUCLK & XTON))						// XT was not enabled
    {
		CLK_CrystalControl( XTON );           	// Enable crystal oscillator
        for ( i = 0; i < 30000; i++);			// Delay
        CLK_SelectClockSource( XTCLOCK );		// Switch Crystal to be clock source
    }
    else
    {
        CLK_SelectClockSource( XTCLOCK );       // Switch Crystal to be clock source
        for ( i = 0; i < 30000; i++);			// Delay
    }
	j++;										// Error count increasd
    if (j >= 10) return (FALSE);				// Fail at Crystal init
  }
  return (TRUE);								// Successful at Crystal init
}

void RC10MCLK_Init ( void )
{
  // Initialize the RC10M Clock to run the system, divided by one (default)
  CLK_RcControl( RCON );             	  //Enable crystal oscillator
  CLK_SelectClockSource( RC10MHZ );       //Selects crystal as clock source
}

//************** Main program *******************
int main()
{
  uint8_t result1, result2;
  
  if (XTCLK_Init()==FALSE)				// Initialize system clock to be the Crystal
    while (1);                                          // Signal clock fail here by blinking a LED
  CLK_RcControl( RCOFF );             	                // Disable RC 10MHz
  
  Port_Config(PTA, 0, 0, 0x04, 0x00, 0x00, 0x00, 0x00); //PA.2 used to signal ADCs conversion completion
  ADC_SelectChannel ( ADC1_PA0, ADC2_PC0);		//Select channels for initial test 
  ADC_ClkDiv (16);					//Select ADCs clock divider (1,2,4,8, or 16)
  ADC_Reference_Config ( ADC1, 15, 4, 7, ADC_REFBG);	//ADC1 referenced to Vdd, range 0 < Vin < (15/15) Vdd
  ADC_Reference_Config ( ADC2, 15, 4, 7, ADC_REFBG);	//ADC2 referenced to Vdd, range 0 < Vin < (15/15) Vdd
  ADC_Enable (ADCEN);									                  //Enable ADCs
  while (1)
  {
	if (XTCLK_Init()==FALSE)								// Check system clock to be the Crystal
		while (1);                                          // Signal clock fail here by a LED   
	ADC_Start ();	                                //Start Conversion
    while ( ADC_ConversionInProgress () );	        //Wait to complete 
    *PORTA ^= 0x04;                                     //Toggle PA.2 
    result1 = ADC_Read (ADC1);				//Read results from both ADCs
    result2 = ADC_Read (ADC2);
  }//while (1)   
}//main()

//************** IRS *******************
void BrownOut_Handler ( void )
{
  
}

void ClkMon_Handler ( void )      		// IRQ 1 clock Monitor
{
  
}

void SS_Handler ( void )          		// IRQ 2 Short Circuit
{
  
}

void PIN_Handler ( void )
{

}

void RFRE_Handler ( void )  		// IRQ 4 RF Rise Edge
{
  
}

void RFFE_Handler ( void )  		// IRQ 5 RF Fall Edge
{
  
}

void I2C_Collision_Handler ( void )  	// IRQ 6 I2C Collision
{
  
}

void I2C_Handler ( void )  		// IRQ 7 I2C
{
  
}

void UART_Handler ( void )  		// IRQ 8 UART
{
  
}

void LIN_Handler ( void )  		// IRQ 9 LIN
{
  
}

void SPI_Handler ( void )               // IRQ A SPI
{
  
}

void US_Handler ( void )  			// IRQ B Ultra-Sound
{
  
}

void RFMSG_Handler ( void )  		// IRQ C RF Message Arrived
{
  
}



void Timer0_Handler( void )
{
  
}

void Timer1_Handler( void )
{
  
}

void Timer2_Handler( void )
{
  
}

void Watchdog_Handler( void )
{
  
}

void Default_IRQ_Handler ( void )
{
  
}


//***************************** End of File ************************************