/** 
 * @brief  Flash program for uSesame
 * @param  -
 * @return -
 * 
 */
#include <stdint.h>
#include "usesame.h"

//************** Definations *******************
typedef struct{ uint16_t write_flag;
                }JOURNEY_PLUS_FLAG_TYPE;
#define JOURNEY_PLUS_FLAG_BASE 0x00027000
#define JOURNEY_PLUS_FLAG ((__IO JOURNEY_PLUS_FLAG_TYPE *) JOURNEY_PLUS_FLAG_BASE)

#define DataOneByte ((__IO uint8_t *)(JOURNEY_PLUS_FLAG_BASE+3))

//************** Global Variables *******************
uint8_t MyRAM[256]={0 , 1, 2, 3, 4, 5, 6, 7, 8, 9,
                    10,11,12,13,14,15,16,17,18,19};
uint16_t learn_count;
uint8_t Data;

//************** Prototype *******************

uint8_t XTCLK_Init ( void );
void RC10MCLK_Init ( void );
void CopyRAM2Flash(uint8_t*, uint8_t*, uint32_t);

//************** Functions *******************

/*
* @param 	RC10KHZ - Clock source = RC AUX 10KHz
* @param 	RC10MHz - Clock source = RC 10MHz
* @param 	XTCLOCK - Clock source = XT       */

uint8_t XTCLK_Init ( void )
{
  int i,j;
  
  // Initialize the Crystal Clock to run the system, divided by one (default)
  j=0;
  if (!(*PMUCLK & (XTCLOCK|RC10MHZ))) CLK_SelectClockSource( RC10MHZ ); // Switch 10K to 10MHz
  while ((*PMUCLK & (XTCLOCK|RC10MHZ)) != XTCLOCK )  		//Wait for XT ON amd stable
  {
    if (!(*PMUCLK & XTON))						// XT was not enabled
    {
		CLK_CrystalControl( XTON );           	// Enable crystal oscillator
        for ( i = 0; i < 30000; i++);			// Delay
        CLK_SelectClockSource( XTCLOCK );		// Switch Crystal to be clock source
    }
    else
    {
        CLK_SelectClockSource( XTCLOCK );       // Switch Crystal to be clock source
        for ( i = 0; i < 30000; i++);			// Delay
    }
	j++;										// Error count increasd
    if (j >= 10) return (FALSE);				// Fail at Crystal init
  }
  return (TRUE);								// Successful at Crystal init
}

void RC10MCLK_Init ( void )
{
  // Initialize the RC10M Clock to run the system, divided by one (default)
  CLK_RcControl( RCON );             	  //Enable crystal oscillator
  CLK_SelectClockSource( RC10MHZ );       //Selects crystal as clock source
}

void CopyRAM2Flash(uint8_t *SourceAddr, uint8_t *DestAddr, uint32_t Size)
{
  uint32_t i;
  uint8_t *pS_Addr, *pD_Addr;
  
  pS_Addr=SourceAddr;
  pD_Addr=DestAddr;
  for (i=0; i<Size; i++)
  {
    FLS_WriteByte((uint32_t)pD_Addr,*pS_Addr);
    pS_Addr ++;                            // Point to next source address
    pD_Addr ++;                            // Point to next destination address
    while(FLS_CheckWritingByte());         //Wait for data writing completion
  }
}

//************** Main program *******************
int main()
{
//  JOURNEY_PLUS_FLAG_TYPE* JOURNEY_PLUS_Flash;
  
  RC10MCLK_Init();
  //FLS_EraseBlock(0x00027000);              //Erase 0x27000 to 0x277FF
  FLS_EraseSector(0x00027000);               //Erase 0x27000 to 0x270FF
  while(FLS_CheckErasingBlock());            //Wait for block erasing completion
  CopyRAM2Flash((uint8_t *)MyRAM,(uint8_t *)0x00027000, sizeof(MyRAM));
  learn_count=JOURNEY_PLUS_FLAG->write_flag;
  Data=*DataOneByte;

  while (1)
  {
  }
}

extern void NMI_Handler( void )
{
  
}

void SysTick_Handler( void )
{
  
}

void BrownOut_Handler ( void )
{
  
}

void ClkMon_Handler ( void )
{
  
}

void SS_Handler ( void )
{
  
}

void RFRE_Handler ( void )              // RF Rising Edge
{
  
}

void RFFE_Handler ( void )              // RF Falling Edge
{
  
}

void UART_Handler( void )  		// IRQ 8 UART
{
  
}

void LIN_Handler( void )  		// IRQ 9 LIN
{
  
}

void SPI_Handler( void )  		// IRQ A SPI
{
  
}

void US_Handler( void )  		// IRQ B Ultra-Sound
{
  
}

void RFMSG_Handler( void )  			// IRQ C - RF Message Arrived
{

}

void Default_IRQ_Handler( void )
{
  
}

void Timer0_Handler( void )
{
  
}

void Timer1_Handler( void )
{
  
}

void Timer2_Handler( void )
{
  
}

void Watchdog_Handler( void )
{
  
}

void I2C_Collision_Handler( void )
{

}

void I2C_Handler( void )
{

}

void PIN_Handler ( void )
{  
}



//***************************** End of File ************************************
