/**
* @brief GPIO Test Software for uSesame
*
* By: CG
* 
*/

#include <stdint.h>
#include "uSesame.h"

//**********************************************************
//********************* Test Defines ***********************
//**********************************************************

//#define TEST_INPUTS_LOW_THRESHOLD
#define TEST_INPUTS_HIGH_THRESHOLD
//#define TEST_OUTPUTS
//#define TEST_INTS


struct GPIO_Error{
  uint8_t writen[5][8];
  uint8_t read[5][8];
}GPIO_Error;

int i = 0,j;
int k,w=0;
uint8_t portvalue,intflag;

uint8_t port_vector[] = {PTA,PTB,PTC,PTD,PTE};
__IO uint8_t *port_pointer;
uint16_t port_nbits[] = {0x100,0x100,0x100,0x100,0x10};

int main()
{
  while (1)
  {
    
// Test all ports as inputs with both pull up and pull down and both threshold levels
//     
#if defined (TEST_INPUTS_LOW_THRESHOLD)
    
    port_pointer = PORTA;
    for (i = 0; i < 5; i++)                           //For all ports
    {
      w = 0;
      for (j = 0x001; j < port_nbits[i]; j *= 2)        //For all I/Os of the port
      {
        //No aux. function, no ints., no output, low current, with pull ups and complementary pull downs, low level
        //PORTD and PORTE require external stimuli (per pin)
        Port_Config(port_vector[i], 0, 0, 0, 0, j, ~j, 0);
        for (k=0;k<1000;k++);                      //Some delay to ensure port is stable
        portvalue =  *port_pointer;
        if (*port_pointer != ((uint8_t)j))
        {
          GPIO_Error.writen[i][w] = j;                 //Code will record error 
          GPIO_Error.read[i][w] = *port_pointer;       
        }
        w++;
      }//for (j = 0x001; j < 0x100; j*=2)         //For all I/Os of the port
      port_pointer++;                             //Point to the next port
    }//for (i = 0; i < 5; i++)                    //For all ports

#elif defined (TEST_INPUTS_HIGH_THRESHOLD)
    
    port_pointer = PORTA;
    for (i = 0; i < 5; i++)                           //For all ports
    {
      w = 0;
      for (j = 0x001; j < port_nbits[i]; j *= 2)        //For all I/Os of the port
      {
        //No aux. function, no ints., no output, low current, with pull downs, high level
        //External stimuli
        Port_Config(port_vector[i], 0, 0, 0, 0, 0, 0xFF, 0xFF);
        for (k=0;k<1000;k++);                      //Some delay to ensure port is stable
        portvalue =  *port_pointer;
        while ( ( *port_pointer & j ) == 0 );
        while ( ( *port_pointer & j ) == j );
        w++;
      }//for (j = 0x001; j < 0x100; j*=2)         //For all I/Os of the port
      port_pointer++;                             //Point to the next port
    }//for (i = 0; i < 5; i++)                    //For all ports
    
    
#elif defined (TEST_OUTPUTS) 

// Test all ports as outputs 
//     
    port_pointer = PORTA;
    for (i = 0; i < 5; i++)                           //For all ports
    {
      w = 0;
      for (j = 0x001; j < port_nbits[i]; j *= 2)        //For all I/Os of the port
      {
        //No aux. function, no ints., low current, no pull ups nor pull downs, low level
        //PORTD and PORTE require external stimuli (per pin)
        Port_Config(port_vector[i], 0, 0, j, 0, 0, 0, 0);
        *port_pointer = j;                              //Test forcing the pin to 1
        for (k=0;k<1000;k++);                           //Some delay to ensure port is stable
        portvalue =  *port_pointer;
        if ( (*port_pointer & ((uint8_t)j)) == 0)
        {
          GPIO_Error.writen[i][w] = j;                 //Code will record error 
          GPIO_Error.read[i][w] = *port_pointer;       
        }
        
        *port_pointer = ~j;                             //Test forcing the pin to 0
        for (k=0;k<1000;k++);                           //Some delay to ensure port is stable
        portvalue =  *port_pointer;
        if ( (*port_pointer & ((uint8_t)j)) == 1)
        {
          GPIO_Error.writen[i][w] = j;                 //Code will record error 
          GPIO_Error.read[i][w] = *port_pointer;       
        }
        
        
        w++;
      }//for (j = 0x001; j < 0x100; j*=2)             //For all I/Os of the port
      port_pointer++;                                 //Point to the next port
    }//for (i = 0; i < 5; i++)                        //For all ports

#elif defined (TEST_INTS)
    
//Port_Config( uint8_t pt, uint8_t ss, uint8_t imask, uint8_t oe, uint8_t st, uint8_t pu, uint8_t pd, uint8_t level )
// Test ports for interrupt on change
//     
    port_pointer = PORTA;
    for (i = 0; i < 3; i++)                             //For PORTA, PORTB and PORTC
    {
      w = 0;
      for (j = 0x001; j < port_nbits[i]; j *= 2)        //For all I/Os of the port
      {
        intflag = 0;                                    //Reset int. flag,to be set inside interrupt
        //No aux. function, interrupt, low current 
        Port_Config(port_vector[i], 0, j, 0, 0, 0, 0xFF, 0);
        for (k=0;k<1000;k++);                           //Some delay to ensure port is stable
        NVIC_EnableIRQ(PIN_IRQn);      
        w++;
        //At this point the tester (person) must raise one pin to test for interrupt
        while (intflag == 0);                           //Wait for interrupt to be served
      }//for (j = 0x001; j < 0x100; j*=2)               //For all I/Os of the port
      port_pointer++;                                   //Point to the next port
    }//for (i = 0; i < 5; i++)                          //For all ports
    
#endif    
    
  }//while (1)   
  
}//main()


void BrownOut_Handler ( void )
{
  
}

void ClkMon_Handler ( void )      		// IRQ 1 clock Monitor
{
  
}

void SS_Handler ( void )          		// IRQ 2 Short Circuit
{
  
}

void PIN_Handler ( void )
{
  while ( *port_pointer & j );    //While pin is not released
  portvalue =  *port_pointer;     //Load current value
  intflag = 1;                    //Flag application that an int. was triggered
}

void RFRE_Handler ( void )  		// IRQ 4 RF Rise Edge
{
  
}

void RFFE_Handler ( void )  		// IRQ 5 RF Fall Edge
{
  
}

void I2C_Collision_Handler ( void )  	// IRQ 6 I2C Collision
{
  
}

void I2C_Handler ( void )  		// IRQ 7 I2C
{
  
}

void UART_Handler ( void )  		// IRQ 8 UART
{
  
}

void LIN_Handler ( void )  		// IRQ 9 LIN
{
  
}

void SPI_Handler ( void )               // IRQ A SPI
{
  
}

void US_Handler ( void )  			// IRQ B Ultra-Sound
{
  
}

void RFMSG_Handler ( void )  		// IRQ C RF Message Arrived
{
  
}



void Timer0_Handler( void )
{
  
}

void Timer1_Handler( void )
{
  
}

void Timer2_Handler( void )
{
  
}

void Watchdog_Handler( void )
{
  
}

void Default_IRQ_Handler ( void )
{
  
}


//***************************** End of File ************************************