/**
* @brief PWM Test Software for uSesame
*
* By: David Chen
* 
*/

#include <stdint.h>
#include "uSesame.h"

//************** Prototype *******************

uint8_t XTCLK_Init ( void );
void RC10MCLK_Init ( void );

//************** Functions *******************

/*
* @param 	RC10KHZ - Clock source = RC AUX 10KHz
* @param 	RC10MHz - Clock source = RC 10MHz
* @param 	XTCLOCK - Clock source = XT       */

uint8_t XTCLK_Init ( void )
{
  int i,j;
  
  // Initialize the Crystal Clock to run the system, divided by one (default)
  j=0;
  if (!(*PMUCLK & (XTCLOCK|RC10MHZ))) CLK_SelectClockSource( RC10MHZ ); // Switch 10K to 10MHz
  while ((*PMUCLK & (XTCLOCK|RC10MHZ)) != XTCLOCK )  		//Wait for XT ON amd stable
  {
    if (!(*PMUCLK & XTON))						// XT was not enabled
    {
		CLK_CrystalControl( XTON );           	// Enable crystal oscillator
        for ( i = 0; i < 30000; i++);			// Delay
        CLK_SelectClockSource( XTCLOCK );		// Switch Crystal to be clock source
    }
    else
    {
        CLK_SelectClockSource( XTCLOCK );       // Switch Crystal to be clock source
        for ( i = 0; i < 30000; i++);			// Delay
    }
	j++;										// Error count increasd
    if (j >= 10) return (FALSE);				// Fail at Crystal init
  }
  return (TRUE);								// Successful at Crystal init
}

void RC10MCLK_Init ( void )
{
  // Initialize the RC10M Clock to run the system, divided by one (default)
  CLK_RcControl( RCON );             	  //Enable RC 10MHz
  CLK_SelectClockSource( RC10MHZ );       //Selects crystal as clock source
}

//************** Main program *******************

int main()
{
  
  if (XTCLK_Init()==FALSE)					  // Initialize system clock to be the Crystal
    while (1);                                                    // Signal clock fail here by a LED
  CLK_RcControl( RCOFF );             	                          // Disable RC 10MHz
  
    TMR_Config(0,TIMERON, -3000000);    //Initialize all three timers
 
  
 NVIC_EnableIRQ(TIMER0_IRQn);      //Enable the interrupts

  //*LEDIO = 0x00;
 // Port_Config( PTB ,(PC3_PWM2), 0, 0x80, 0, 0, 0, 0);             //PB0 as output, PWM1 enabled
    

    Port_Config(PTE,0, 0, 0xFF, 0, 0, 0, 0);              
  *PORTE = 0xFF;

  Port_Config(PTA,0, 0, 0xFF, 0, 0x00, 0x00, 0);              
  *PORTA = 0xFF;

  Port_Config(PTB,0, 0, 0xFF, 0, 0, 0, 0);              
  *PORTB = 0xFF;

  Port_Config(PTD,0, 0, 0xFF, 0, 0, 0, 0);              
  *PORTD = 0xFF; 
  Port_Config(PTC,PC3_PWM2, 0, 0xFF, 0, 0, 0, 0);              
  *PORTC = 0xFF;

 
   

    
  PWM_Setup(PWM2,PWMEN, PWMNINV, PRESC0);
  PWM_Period (PWM2, 700);
  PWM_Width (PWM2, 50);
  while (1)
  {
    if (XTCLK_Init()==FALSE)					  // Check system clock to be the Crystal
      while (1);                                                  // Signal clock fail here by a LED    
  }
    
}//main()


void BrownOut_Handler ( void )
{
  
}

void ClkMon_Handler ( void )      		// IRQ 1 clock Monitor
{
  
}

void SS_Handler ( void )          		// IRQ 2 Short Circuit
{
  
}

void PIN_Handler ( void )
{

}

void RFRE_Handler ( void )  		// IRQ 4 RF Rise Edge
{
  
}

void RFFE_Handler ( void )  		// IRQ 5 RF Fall Edge
{
  
}

void I2C_Collision_Handler ( void )  	// IRQ 6 I2C Collision
{
  
}

void I2C_Handler ( void )  		// IRQ 7 I2C
{
  
}

void UART_Handler ( void )  		// IRQ 8 UART
{
  
}

void LIN_Handler ( void )  		// IRQ 9 LIN
{
  
}

void SPI_Handler ( void )               // IRQ A SPI
{
  
}

void US_Handler ( void )  			// IRQ B Ultra-Sound
{
  
}

void RFMSG_Handler ( void )  		// IRQ C RF Message Arrived
{
  
}



void Timer0_Handler( void )
{
  
    *TMR0REG = -3000000;   		//Reload Register  
    *PORTE ^= 0x0F;                       
    *PORTA ^= 0xFF;    
    *PORTB ^= 0xFF;
    *PORTC ^= 0xFF;
    *PORTD ^= 0xFF;
   
}

void Timer1_Handler( void )
{
  
}

void Timer2_Handler( void )
{
  
}

void Watchdog_Handler( void )
{
  
}

void Default_IRQ_Handler ( void )
{
  
}


//***************************** End of File ************************************