/** 
 * @brief  I2C Slave Test Program for uSOBEK
 * @param  -
 * @return -
 * Receives a message of up to 
 */

#include <stdint.h>
#include "uSesame.h"
#include "RollingCode.h"

//************** Defination *******************
//#define DEBUG
#define CORRECT_FRAME_SIZE  65

#define ID 0x37B4F5
#define Button 2

//************** Global variables *******************				  
uint8_t rxbuffer[10];                   //RF buffer 


//PROTOTYPES

//************** Prototype *******************

uint8_t XTCLK_Init ( void );
void RC10MCLK_Init ( void );
void Read_Buffer (uint32_t *);
void RF_Init (void);

//************** Functions *******************

/*
* @param 	RC10KHZ - Clock source = RC AUX 10KHz
* @param 	RC10MHz - Clock source = RC 10MHz
* @param 	XTCLOCK - Clock source = XT       */

uint8_t XTCLK_Init ( void )
{
  int i,j;
  
  // Initialize the Crystal Clock to run the system, divided by one (default)
  j=0;
  if (!(*PMUCLK & (XTCLOCK|RC10MHZ))) CLK_SelectClockSource( RC10MHZ ); // Switch 10K to 10MHz
  while ((*PMUCLK & (XTCLOCK|RC10MHZ)) != XTCLOCK )  		//Wait for XT ON amd stable
  {
    if (!(*PMUCLK & XTON))						// XT was not enabled
    {
		CLK_CrystalControl( XTON );           	// Enable crystal oscillator
        for ( i = 0; i < 30000; i++);			// Delay
        CLK_SelectClockSource( XTCLOCK );		// Switch Crystal to be clock source
    }
    else
    {
        CLK_SelectClockSource( XTCLOCK );       // Switch Crystal to be clock source
        for ( i = 0; i < 30000; i++);			// Delay
    }
	j++;										// Error count increasd
    if (j >= 10) return (FALSE);				// Fail at Crystal init
  }
  return (TRUE);								// Successful at Crystal init
}
  
void RC10MCLK_Init ( void )
{
  // Initialize the RC10M Clock to run the system, divided by one (default)
  CLK_RcControl( RCON );             	  //Enable crystal oscillator
  CLK_SelectClockSource( RC10MHZ );       //Selects crystal as clock source
}


void Read_Buffer ( uint32_t *RxBuffer )
{
  uint8_t i, rxbuffer[10];

  for ( i = 0; i < 10; i++)           //Load msg. into rxbuffer moving byte9 <--> byte0, byte8 <--> byte1, etc.
  {
    rxbuffer[i] = RXDATA->BYTE[9-i]; 
  }
  RxBuffer[0] = (rxbuffer[5]<<24) + (rxbuffer[4]<<16) + (rxbuffer[3]<<8) + (rxbuffer[2]);
  RxBuffer[1] = (rxbuffer[9]<<24) + (rxbuffer[8]<<16) + (rxbuffer[7]<<8) + (rxbuffer[6]);
}

void RF_Init ( void )
{
  // RF Setup for 433.92MHz reception
  
  RF_SetState( RFEN );                  //RF Enabled
  RF_SnifferMode( SNIFEN );             //Sniffer enabled
  RF_AgcControl( AGCEN, 0x1A );         //AGC Enabled
  RF_SetPllBiasTime( PLL48 );           //PLL waits 48 clock cycles 
  RF_SetSleepTime( RFSLEEP10 );         //Sleep time for sniffer is 10*1024 cycles
  RF_SetWakeTime( WAKET18 );            //Wake time from sniff is 18*1024 cycles 
  *RFNX = 0x79;                         //PLL Integer divider
  *RFNF = 0x00;                         //PLL Fractional Divider
  // Hi side rejection enabled, fractional disabled, charge pump and loop filter
  RF_Setup0( HISDEN, FDIS, CPT01, 0x0A ); 
  //16 +DIV_LO2[1:0], Base band gain 00 = 0dB, LNA drain res. 1Kohm, LNA bias 1mA
  RF_Setup1( DIV_LO2_18, BBGAIN0dB, LNADR1K, LNABIAS1mA	 );  
  //Delay timer of the slicer, Attack time of the slicer, Time to allow slicer 
  //to fast bias, Symbol Decimation rate
  RF_SlicerControl( ALPHA02, BETA02 , FT860uS , SDR03 );      
  RF_SetMinTe( 3 );                     //(1+Min_Te)*(12*16)/3.58e6 
  RF_SetMaxTe( 0x1B );                  //(1 + Min_Te + Max_Te)*(12*16)/3.58e6
  RF_SetMinGb( 0x07 );                  //(1 + Min_Te + Max_Te + Min_Gb)*(12*16)/3.58e6
  RF_SetSnifRt( RTPLL, RFONDIS, 0x0B );
  RF_SetMinBitNumber( 0x28 );
  
}
uint16_t Counter;
//************** Main Entry *******************
int main()
{  
  if (!XTCLK_Init())
    while (1);      // Signal LED to indect the system clack failure
  
#ifdef DEBUG
CodeTest:
#endif
  Counter=100;
  //Encrypt ID & buttom #
  Encrypt_Message(ID,Button,Vec); //input:ID, buttom #; output:Vec
  //Decrypt ID and buttom
  if ( Decrypt_Message(&rf_msg,Vec) == TRUE )
  {
    if ( rf_msg.id == ID )
    {
      RF_Init();          //Initialize RF reception     
    }  
  }
#ifdef DEBUG
  goto CodeTest;
#endif
  NVIC_EnableIRQ( RFMSG_IRQn );         //Enable RF reception interrupt
  while ( 1 );
  
}

//******************* ISR ***********************
extern void NMI_Handler( void )
{
  
}

void SysTick_Handler( void )
{
  
}

void BrownOut_Handler ( void )
{
  
}

void ClkMon_Handler ( void )
{
  
}

void SS_Handler ( void )
{
  
}

void RFRE_Handler ( void )              // RF Rising Edge
{
  
}

void RFFE_Handler ( void )              // RF Falling Edge
{
  
}

void UART_Handler( void )  		// IRQ 8 UART
{
  
}

void LIN_Handler( void )  		// IRQ 9 LIN
{
  
}

void SPI_Handler( void )  		// IRQ A SPI
{
  
}

void RFMSG_Handler( void )  		// IRQ C RF Message Arrived
{
  if ( *RXNUMBER <= (CORRECT_FRAME_SIZE+4) )// If received correct number of bits
  {
    Read_Buffer(Vec);
    if( Decrypt_Message(&rf_msg,Vec) == FALSE )            // Decode Message
      rf_msg.id=0;
  }
  RF_SetState(RFEN);                    // Set the RF on
  RF_ReinitDecoder();                   // Clear the MSGRDY bit, restarting reception
  
}

void Default_IRQ_Handler( void )
{
  
}

void Timer0_Handler( void )
{

}

void Timer1_Handler( void )
{
  
}

void Timer2_Handler( void )
{
  
}

void Watchdog_Handler( void )
{
  
}

void I2C_Collision_Handler( void )
{

}

void I2C_Handler( void )
{

}

void PIN_Handler ( void )
{
  
}
//***************************** End of File ************************************