/**
* @Simple example of using uSesame SPI
* SPI loopback test. Please wires MOSI and MISO pins
*
* 4/26/2012
* By: David Chen
* 
*/

#include <stdint.h>
#include "uSesame.h"

//************** Defination *******************


//************** Global variables *******************
uint8_t SPI_OutData[12]={0x55, 0xAA, 0x33, 0xCC,
                0x01, 0x02, 0x03, 0x04,
                0x05, 0x06, 0x07, 0x08};
uint8_t mydata[20];

//************** Prototype *******************
void RC10MCLK_Init ( void );
uint8_t XTCLK_Init ( void );
void SPI_Init(uint32_t);

//************** Functions *******************
/*
* @param 	RC10KHZ - Clock source = RC AUX 10KHz
* @param 	RC10MHz - Clock source = RC 10MHz
* @param 	XTCLOCK - Clock source = XT       */

uint8_t XTCLK_Init ( void )
{
  int i,j;
  
  // Initialize the Crystal Clock to run the system, divided by one (default)
  j=0;
  if (!(*PMUCLK & (XTCLOCK|RC10MHZ))) CLK_SelectClockSource( RC10MHZ ); // Switch 10K to 10MHz
  while ((*PMUCLK & (XTCLOCK|RC10MHZ)) != XTCLOCK )  		//Wait for XT ON amd stable
  {
    if (!(*PMUCLK & XTON))						// XT was not enabled
    {
		CLK_CrystalControl( XTON );           	// Enable crystal oscillator
        for ( i = 0; i < 30000; i++);			// Delay
        CLK_SelectClockSource( XTCLOCK );		// Switch Crystal to be clock source
    }
    else
    {
        CLK_SelectClockSource( XTCLOCK );       // Switch Crystal to be clock source
        for ( i = 0; i < 30000; i++);			// Delay
    }
	j++;										// Error count increasd
    if (j >= 10) return (FALSE);				// Fail at Crystal init
  }
  return (TRUE);								// Successful at Crystal init
}
  
void RC10MCLK_Init ( void )
{
  // Initialize the RC10M Clock to run the system, divided by one (default)
  CLK_RcControl( RCON );             	  //Enable crystal oscillator
  CLK_SelectClockSource( RC10MHZ );       //Selects crystal as clock source
}

void SPI_Init(uint32_t DIV) 
{
  if (XTCLK_Init()==FALSE)				// Initialize system clock to be the Crystal
    while (1);                                          // Signal clock fail here by blinking a LED
  *PORTDOE |= MODESPI;                                  // Switch the IO pins to be SPI pins
  SPI_Config(SPI_INT_EN, SPI_EN, SPI_CPOL_NEG, SPI_PHASE0, DIV);
  NVIC_ClearPendingIRQ(SPI_IRQn);
  NVIC_EnableIRQ(SPI_IRQn);
}

//************** Main program *******************
void main (void)
{
  uint8_t i;
 
  RC10MCLK_Init();
  
  SPI_Init(32);
  i=0;
  while (1)
  {
    if (XTCLK_Init()==FALSE)				// Check system clock to be the Crystal
      while (1);                                        // Signal system freq. fail
    if ((!(SPI_ReadStatus() & SPI_TX_FIFO_FULL)) && (i < (sizeof(SPI_OutData)/sizeof(SPI_OutData[0]))))
        *SPIDATA=SPI_OutData[i++];
  }//while (1)   
}//main()

//**********************************************
//ISRs
//**********************************************
extern void NMI_Handler( void )
{
  
}

void SysTick_Handler( void )
{
  
}

void BrownOut_Handler ( void )
{
  
}

void ClkMon_Handler ( void )
{
  
}

void SS_Handler ( void )
{
  
}

void RFRE_Handler ( void )              // RF Rising Edge
{
  
}

void RFFE_Handler ( void )              // RF Falling Edge
{
  
}

void UART_Handler( void )  		// IRQ 8 UART
{

}

void LIN_Handler( void )  		// IRQ 9 LIN
{
  
}

void SPI_Handler( void )  		// IRQ A SPI
{
   __IO uint8_t status;
  static uint8_t i=0;

  while (!(SPI_ReadStatus() & SPI_RX_FIFO_EMPTY))
        mydata[i++] = *SPIDATA;
  status = SPI_ReadStatus();          // This is not real time for following test conditions
    if (status & SPI_WCOL)
    {
      //Process write collision (data is written to the SPI data
      //register while a SPI data transfer is in progress)
    }
    if (status & SPI_TX_FIFO_FULL)
    {
      //Process Transmission FIFO full
    }
    if (status & SPI_RX_FIFO_FULL)      // This should not happen
    {
      //Process reception FIFO full, normally by reading
      //all bytes of data
      while (!(SPI_ReadStatus() & SPI_RX_FIFO_EMPTY))
        mydata[i++] = *SPIDATA;
    }
    if (status & SPI_TX_FIFO_EMPTY)
    {
      //Process end of transmission of data previously
      //in transmission FIFO
/*
      *SPIDATA = 0x01;
      *SPIDATA = 0x02;
      *SPIDATA = 0x03;
      *SPIDATA = 0x04;
*/
    }
    //Other processing here
    if (status & SPI_RX_FIFO_EMPTY)
    {
      //Process when no more information is available (received)
    }
//  }
  SPI_INT_Clear();  // Clear SPI INT flag to accept next interrupt
  if (i > (sizeof(mydata)/sizeof(mydata[0]))) i=0;
}

void US_Handler( void )  		// IRQ B Ultra-Sound
{
  
}

void RFMSG_Handler( void )  			// IRQ C - RF Message Arrived
{
 
}

void Default_IRQ_Handler( void )
{
  
}

void Timer0_Handler( void )
{

}

void Timer1_Handler( void )
{
  
}

void Timer2_Handler( void )
{
  
}

void Watchdog_Handler( void )
{
  
}

void I2C_Collision_Handler( void )
{

}

void I2C_Handler( void )
{

}

void PIN_Handler ( void )
{

}


//***************************** End of File ************************************