/*
Warning: This file was modified to be used along with AyDeeKay's uSesame
device.
by: CG - 2/3/2011, modified on 6/16/2011
*/

/**************************************************
 *
 * This file contains an interrupt vector for Cortex-M written in C.
 * The actual interrupt functions must be provided by the application developer.
 *
 * Copyright 2007 IAR Systems. All rights reserved.
 *
 * $Revision: 25622 $
 *
 **************************************************/

#pragma language=extended
#pragma segment="CSTACK"

extern void __iar_program_start( void );
// extern void main( void );

extern void NMI_Handler( void );
extern void HardFault_Handler( void );
extern void MemManage_Handler( void );
extern void BusFault_Handler( void );
extern void UsageFault_Handler( void );
extern void SVC_Handler( void );
extern void DebugMon_Handler( void );
extern void PendSV_Handler( void );
extern void SysTick_Handler( void );

#if 0
void BrownOut_Handler ( void );         // IRQ 0
void ClkMon_Handler ( void );           // IRQ 1
void SS_Handler ( void );               // IRQ 2
void PIN_Handler ( void );              // IRQ 3
void RFRE_Handler ( void );             // IRQ 4
void RFFE_Handler ( void );             // IRQ 5
void I2C_Collision_Handler( void ); 	// IRQ 6
void I2C_Handler( void ); 		        // IRQ 7
void UART_Handler( void );  		    // IRQ 8 UART
void LIN_Handler( void );  		        // IRQ 9 LIN
void SPI_Handler( void );  		        // IRQ A SPI
void RFMSG_Handler( void );  		    // IRQ C RF Message Arrived
void Default_IRQ_Handler( void );
void Timer0_Handler( void );
void Timer1_Handler( void );
void Timer2_Handler( void );
void Watchdog_Handler( void );
extern void BTE_Handler( void );
extern void SDIO_Handler( void );
#else
extern void BrownOut_Handler ( void );          // IRQ 0
extern void ClkMon_Handler ( void );            // IRQ 1
extern void SS_Handler ( void );                // IRQ 2
extern void PIN_Handler ( void );               // IRQ 3
extern void RFRE_Handler ( void );              // IRQ 4
extern void RFFE_Handler ( void );              // IRQ 5
extern void I2C_Collision_Handler( void ); 	    // IRQ 6
extern void I2C_Handler( void ); 		        // IRQ 7
extern void UART_Handler( void );  		        // IRQ 8 UART
extern void LIN_Handler( void );  		        // IRQ 9 LIN
extern void SPI_Handler( void );  		        // IRQ A SPI
extern void RFMSG_Handler( void );  		    // IRQ C RF Message Arrived
extern void Default_IRQ_Handler( void );
extern void Timer0_Handler( void );
extern void Timer1_Handler( void );
extern void Timer2_Handler( void );
extern void Watchdog_Handler( void );
extern void BTE_Handler( void );
extern void SDIO_Handler( void );
#endif

typedef void( *intfunc )( void );
typedef union { intfunc __fun; void * __ptr; } intvec_elem;

// The vector table is normally located at address 0.
// When debugging in RAM, it can be located in RAM, aligned to at least 2^6.
// If you need to define interrupt service routines,
// make a copy of this file and include it in your project.
// The name "__vector_table" has special meaning for C-SPY:
// it is where the SP start value is found, and the NVIC vector
// table register (VTOR) is initialized to this address if != 0.

#pragma location = ".intvec"
const intvec_elem __vector_table[] =
{
  { .__ptr = __sfe( "CSTACK" ) },
  __iar_program_start,

  NMI_Handler,
  HardFault_Handler,
  MemManage_Handler,
  BusFault_Handler,
  UsageFault_Handler,
  0,
  0,
  0,
  0,
  SVC_Handler,
  DebugMon_Handler,
  0,
  PendSV_Handler,
  SysTick_Handler,
  BrownOut_Handler,    		// IRQ 0 Brownout detector
  ClkMon_Handler,      		// IRQ 1 clock Monitor
  SS_Handler,          		// IRQ 2 Short Circuit
  PIN_Handler,       		// IRQ 3 Pin Change 
  RFRE_Handler,  		    // IRQ 4 RF Rise Edge
  RFFE_Handler,  		    // IRQ 5 RF Fall Edge
  I2C_Collision_Handler,  	// IRQ 6 I2C Collision
  I2C_Handler,  		    // IRQ 7 I2C
  UART_Handler,  		    // IRQ 8 UART
  LIN_Handler,  		    // IRQ 9 LIN
  SPI_Handler,  		    // IRQ A SPI
  Default_IRQ_Handler,
  RFMSG_Handler,  		    // IRQ C RF Message Arrived
  Default_IRQ_Handler,  	// IRQ D
  Default_IRQ_Handler,  	// IRQ E
  Default_IRQ_Handler,  	// IRQ F
  Timer0_Handler,
  Timer1_Handler,
  Timer2_Handler,
  Watchdog_Handler,
  BTE_Handler,
  SDIO_Handler,
  Default_IRQ_Handler,         // No harware implemented
  Default_IRQ_Handler,         // No harware implemented
  Default_IRQ_Handler,         // No harware implemented
  Default_IRQ_Handler,         // No harware implemented
  Default_IRQ_Handler,         // No harware implemented
  Default_IRQ_Handler,         // No harware implemented
  Default_IRQ_Handler,         // No harware implemented
  Default_IRQ_Handler,         // No harware implemented
  Default_IRQ_Handler,         // No harware implemented
  Default_IRQ_Handler          // No harware implemented
};


__weak void NMI_Handler( void ) { }
__weak void HardFault_Handler( void ) { }
__weak void MemManage_Handler( void ) { }
__weak void BusFault_Handler( void ) { }
__weak void UsageFault_Handler( void ) { }
__weak void SVC_Handler( void ) { }
__weak void DebugMon_Handler( void ) { }
__weak void PendSV_Handler( void ) { }
__weak void SysTick_Handler( void ) { }
__weak void BTE_Handler( void ) { }
__weak void SDIO_Handler( void ) { }
__weak void US_Handler ( void ) { }


void __cmain( void );
__weak void __iar_init_core( void );
__weak void __iar_init_vfp( void );

#pragma required=__vector_table
void __iar_program_start( void )
{
  __iar_init_core();
  __iar_init_vfp();
  __cmain();
}
