/**
* @brief Timers Test Software for uSesame
*
* By: CG
* 
*/

#include <stdint.h>
#include "uSesame.h"

//**********************************************************
//********************* Test Defines ***********************
//**********************************************************

uint8_t XTCLK_Init ( void )
{
  int i,j;
  
  // Initialize the Crystal Clock to run the system, divided by one (default)
  j=0;
  if (!(*PMUCLK & (XTCLOCK|RC10MHZ))) CLK_SelectClockSource( RC10MHZ ); // Switch 10K to 10MHz
  while ((*PMUCLK & (XTCLOCK|RC10MHZ)) != XTCLOCK )  		//Wait for XT ON amd stable
  {
    if (!(*PMUCLK & XTON))						// XT was not enabled
    {
		CLK_CrystalControl( XTON );           	// Enable crystal oscillator
        for ( i = 0; i < 30000; i++);			// Delay
        CLK_SelectClockSource( XTCLOCK );		// Switch Crystal to be clock source
    }
    else
    {
        CLK_SelectClockSource( XTCLOCK );       // Switch Crystal to be clock source
        for ( i = 0; i < 30000; i++);			// Delay
    }
	j++;										// Error count increasd
    if (j >= 10) return (FALSE);				// Fail at Crystal init
  }
  return (TRUE);								// Successful at Crystal init
}

int main()
{
    if (XTCLK_Init()==FALSE)				// Initialize system clock to be the Crystal
    while (1);                                          // Signal clock fail here by blinking a LED
  Port_Config(PTC, 0, 0, 0x20, 0x00, 0x00, 0x00, 0x00); //PC.0, 1 and 2 used to signal interrupt

  TMR_Config(0,TIMERON, -1000000);                      //Initialize all three timers

  
  NVIC_EnableIRQ(TIMER0_IRQn);                          //Enable the interrupts

  
  while(1)
  {
        if (XTCLK_Init()==FALSE)				// Check system clock to be the Crystal
      while (1);                                        // Signal clock fail here by a LED  
  }
  
}


void BrownOut_Handler ( void )
{
  
}

void ClkMon_Handler ( void )      	// IRQ 1 clock Monitor
{
  
}

void SS_Handler ( void )          	// IRQ 2 Short Circuit
{
  
}

void PIN_Handler ( void )
{

}

void RFRE_Handler ( void )  		// IRQ 4 RF Rise Edge
{
  
}

void RFFE_Handler ( void )  		// IRQ 5 RF Fall Edge
{
  
}

void I2C_Collision_Handler ( void )  	// IRQ 6 I2C Collision
{
  
}

void I2C_Handler ( void )  		// IRQ 7 I2C
{
  
}

void UART_Handler ( void )  		// IRQ 8 UART
{
  
}

void LIN_Handler ( void )  		// IRQ 9 LIN
{
  
}

void SPI_Handler ( void )               // IRQ A SPI
{
  
}

void US_Handler ( void )  			// IRQ B Ultra-Sound
{
  
}

void RFMSG_Handler ( void )  		// IRQ C RF Message Arrived
{
  
}



void Timer0_Handler( void )
{
  *TMR0REG = -1000000;   		//Reload Register  
  *PORTC ^= 0x20;                       //Toggle PC.0
  
}

void Timer1_Handler( void )
{

}

void Timer2_Handler( void )
{

}

void Watchdog_Handler( void )
{

}

void Default_IRQ_Handler ( void )
{
  
}


//***************************** End of File ************************************